#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.Functions.sensor_data import get_einheit
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *

'''Gibt den Sensorwerteverlauf für den gewünschten Zeitraum zurück'''
def get_graph_data(username, room, type, id, von, bis, db):

    if not has_permission(username, room, db):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    if von == bis:
        datum = datetime.datetime.strptime(von, "%d.%m.%Y").strftime("%Y-%m-%d")

        return get_day_data(type, id, datum, db)
    else:
        return get_day_min_max(type, id, von, bis, db)

'''Gibt den Tageswerteverlauf des Sensors zurück'''
def get_day_data(type, id, datum, db):
    with db:
        cur = db.cursor()
        cur.execute("""SELECT * FROM 'SENSOR_DATA' WHERE DEVICE_TYPE == :type AND DEVICE_ID == :id 
                AND DATETIME >= :start AND DATETIME < :ende AND VALUE != \"N/A\" ORDER BY DATETIME ASC""",
                    {'type':type, 'id':id, 'start':datum+" 00:00", 'ende':datum+" 23:59"})
        values = []

        for data in cur.fetchall():
            item = {'value':float(data['VALUE']), 'id':id, 'time':data['DATETIME'].replace(datum+" ","")}
            values.append(item)

        return {'values': values, 'einheit': get_einheit(type, id, db)}

'''Gibt die Höchst- & Tiefstwerte des angegebenen Datumsbereichs zurück'''
def get_day_min_max(type, id, von, bis, db):
    start = datetime.datetime.strptime(von, "%d.%m.%Y").strftime("%Y-%m-%d")
    ende = datetime.datetime.strptime(bis, "%d.%m.%Y").strftime("%Y-%m-%d")

    values = []

    with db:
        cur = db.cursor()
        cur.execute(
            """SELECT MIN(VALUE) as MIN, MAX(VALUE) as MAX, strftime(\"%d.%m.%Y\", DATETIME) as FORMATTED_DATE
             FROM SENSOR_DATA WHERE DEVICE_TYPE == :type AND DEVICE_ID == :id AND DATETIME >= :start 
             AND DATETIME <= :ende AND VALUE != \"N/A\" GROUP BY DATETIME ORDER BY DATETIME ASC""",
            {'type': type, 'id': id, 'start': start+' 00:00', 'ende': ende+' 23:59'})

        #print {'type': type, 'id': id, 'start': start, 'ende': ende}

        for data in cur.fetchall():
            item = {'date':data['FORMATTED_DATE'], 'min':data['MIN'], 'max':data['MAX']}

            values.append(item)

        return {'values':values, 'einheit': get_einheit(type, id, db)}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Functions import sensor_data
from homevee.Functions.sensor_data import SENSOR_TYPE_MAP
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule
from homevee.VoiceAssistant.context_keys import *
from homevee.VoiceAssistant.voice_patterns import PATTERN_SENSOR_DATA
from homevee.utils.device_types import *

class VoiceDeviceGetSensorDataModule(VoiceDeviceControlModule):
    def get_pattern(self, db):
        return PATTERN_SENSOR_DATA

    def get_label(self):
        return "sensor"

    def run_command(self, username, text, context, db):
        return self.get_sensor_data(username, text, context, db)

    def get_context_key(self):
        return CONTEXT_GET_SENSOR_DATA

    def get_sensor_data(self, username, text, context, db):
        room = self.find_room(text, db)

        sensor_type = self.find_sensor_type(text)

        if sensor_type is None:
            if context is not None and context:
                sensor_type = context['sensor_type']

        if room is None:
            if context is None:
                answer_data = [
                    ['Welchen Raum ', ['meinst du', 'möchtest du abfragen'], '?']
                ]
                output = helper.generate_string(answer_data)

                helper.set_context(username, CONTEXT_GET_SENSOR_DATA, {'sensor_type': sensor_type}, db)

                return {'msg_speech': output, 'msg_text': output}
            else:
                if 'room' in context:
                    room = context['room']
                    print(room)

        if sensor_type in SENSOR_TYPE_MAP:
            einheit = SENSOR_TYPE_MAP[sensor_type]['einheit_word']
        else:
            answer_data = [
                ['Diesen Wert kann ich noch nicht abfragen.'],
                ['Dieser Wert kann noch nicht abgefragt werden.']
            ]
            output = helper.generate_string(answer_data)
            return {'msg_speech': output, 'msg_text': output}


        values = []

        with db:
            cur = db.cursor()

            data = [
                {'type': ZWAVE_SENSOR, 'table': 'ZWAVE_SENSOREN', 'id_col': 'ID', 'room_col': 'RAUM', 'type_col': 'SENSOR_TYPE'},
                {'type': MQTT_SENSOR, 'table': 'MQTT_SENSORS', 'id_col': 'ID', 'room_col': 'ROOM', 'type_col': 'TYPE'}
            ]

            for item in data:
                cur.execute("SELECT * FROM "+item['table']+" WHERE "+item['type_col']+" = :type AND "+item['room_col']+" = :room",
                            {'type': sensor_type, 'room': room['LOCATION']})

                for sensor in cur.fetchall():
                    values.append(sensor_data.get_sensor_data(username, room['LOCATION'], item['type'], sensor[item['id_col']], False, db))

        if len(values) > 0:
            #compute average value of all sensors in this room
            sum = 0

            try:
                for value in values:
                    sum += float(value)
            except Exception:
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' gibt es noch keine ', ['Messungen', 'Werte', 'Sensorwerte'],'.']
                ]

                output = helper.generate_string(answer_data)

                return {'msg_speech': output, 'msg_text': output}

            if sensor_type not in ['powermeter']:
                avg_value = sum/len(values)
                value_word = str(avg_value) + ' ' + einheit
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' hat es ' + value_word, '.']
                ]
            else:
                value_word = str(int(sum)) + ' ' + einheit
                answer_data = [
                    ['Im Raum ' + room['NAME'], ' liegt der  ', ['Verbrauch', 'Stromverbrauch'],' bei ', value_word, '.'],
                    ['Der Raum ' + room['NAME'], ' verbraucht aktuell  ', value_word, '.']
                ]

            output = helper.generate_string(answer_data)

            # helper.set_context(username, GET_SENSOR_DATA, {'room': room}, db)
        else:
            answer_data = [
                ['Ich konnte ', ['hier', 'in diesem Raum'] ,' keine ', ['Daten', 'Sensoren', 'Sensorwerte'], ' finden.']
            ]
            output = helper.generate_string(answer_data)

        return {'msg_speech': output, 'msg_text': output}

    def get_reed_data(self, username, text, context, db):
        room = self.find_room(text, db)

        output = 'Reed'
        return {'msg_speech': output, 'msg_text': output}

    def get_presence_data(self, username, text, context, db):
        room = self.find_room(text, db)

        output = 'Anwesenheit'
        return {'msg_speech': output, 'msg_text': output}

    def find_sensor_type(self, text):
        words = text.split()

        type_keywords = [
            {'type': 'temp', 'items': ['warm', 'kalt', 'temperatur']},
            {'type': 'hygro', 'items': ['feucht', 'luftfeuchtigkeit', 'luftfeuchte']},
            {'type': 'helligkeit', 'items': ['hell', 'helligkeit']},
            {'type': 'uv', 'items': ['uv-licht']},
            {'type': 'powermeter', 'items': ['verbrauch', 'stromverbrauch']}
        ]

        for type_keyword in type_keywords:
            for keyword in type_keyword['items']:
                if keyword in words:
                    return type_keyword['type']

        return None


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.items.Status import *

def get_place(id, db):
    if(id is None or id == "" or id is -1):
        return None

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PLACES WHERE ID = :id",
                    {'id': id})

        place = cur.fetchone()

        return place

def get_my_places(username, db):
    with db:
        cur = db.cursor()

        places = []

        cur.execute("SELECT * FROM PLACES")

        for place in cur.fetchall():
            places.append({'id': place['ID'], 'name': place['NAME'], 'address': place['ADDRESS'], 'latitude': place['LATITUDE'], 'longitude': place['LONGITUDE']})

        return {'places': places}

def add_edit_place(username, id, name, address, latitude, longitude, db):
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if (add_new):
            cur.execute("INSERT INTO PLACES (NAME, ADDRESS, LATITUDE, LONGITUDE) VALUES (:name, :address, :latitude, :longitude)",
                        {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude})

            return Status(type=STATUS_OK).get_dict()

        else:
            cur.execute("UPDATE SCENES SET NAME = :name, ADDRESS = :address, LATITUDE = :latitude, LONGITUDE = :longitude WHERE ID = :id",
                        {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude, 'id': id})

            return Status(type=STATUS_OK).get_dict()

def delete_place(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM PLACES WHERE ID = :id", {'id': id})

    return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Helper import Logger
from homevee.items.Status import *


def get_calendar_item_dates(user, year, db):
    calendar_item_dates = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT DISTINCT DATE FROM CALENDAR WHERE strftime('%Y', DATE) = :year",
                    {'year': year})

        for calendar_entry in cur.fetchall():
            calendar_item_dates.append(calendar_entry['DATE'])

    return {'dates': calendar_item_dates}

def get_calendar_day_items(user, date, db):
    calendar_items = []

    with db:
        cur = db.cursor()

        params = {'date': date}

        cur.execute("SELECT * FROM CALENDAR WHERE DATE = :date ORDER BY START ASC", params)

        for calendar_entry in cur.fetchall():
            item = {'name': calendar_entry['NAME'], 'id': calendar_entry['ID'], 'start': calendar_entry['START'],
                    'end': calendar_entry['END'], 'note': calendar_entry['NOTE'], 'address': calendar_entry['ADDRESS']}
            calendar_items.append(item)

    return {'calendar_entries': calendar_items}

def delete_entry(user, id, db):
    return Status(type=STATUS_OK).get_dict().get_dict()

def add_edit_entry(user, entry_id, name, date, start, end, note, address, db):
    entry_id = int(entry_id)

    with db:
        cur = db.cursor()

        params = {'name': name, 'date': date, 'start': start, 'end': end, 'note': note, 'address': address}

        if entry_id is None or entry_id == -1:
            #Create new
            Logger.log("insert")
            cur.execute("INSERT INTO CALENDAR (NAME, START, END, NOTE, DATE, ADDRESS) VALUES (:name, :start, :end, :note, :date, :address)",
                        params)
        else:
            #Edit existing
            Logger.log("update")

            params['id'] = entry_id

            cur.execute("UPDATE CALENDAR SET NAME = :name, START = :start, END = :end, NOTE = :note, DATE = :date, ADDRESS = :address WHERE ID = :id",
                        params)

    return Status(type=STATUS_OK).get_dict()


import traceback

from homevee.Helper import Logger
from homevee.exceptions import ItemNotFoundException, DatabaseSaveFailedException, InvalidParametersException
from homevee.items import Item
from homevee.utils import database


class Event(Item):
    def __init__(self, text, type, timestamp=None, id=None):
        super(Event, self).__init__()

        self.timestamp = timestamp
        self.text = text
        self.type = type
        self.id = id

    def delete(self, db=None):
        try:
            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM EVENTS WHERE ID == :id'", {'id': self.id})

                cur.close()
            return True
        except:
            return False

    def save_to_db(self, db=None):
        try:
            if (db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                # insert
                if (self.id is None or self.id == ""):
                    cur.execute("""INSERT INTO EVENTS (TEXT, TYPE) VALUES (:text, :type)""",
                                {'text': self.text, 'type': self.type})
                # update
                else:
                    cur.execute("""UPDATE EVENTS SET TEXT = :text, TYPE = :type WHERE ID = :id""",
                                {'text': self.text, 'type': self.type, 'id': self.id})

                cur.close()

                #TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save event to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'timestamp': self.timestamp,
            'text': self.text,
            'type': self.type
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)
            for result in cur.fetchall():
                item = Event(result['TEXT'], result['TYPE'], result['TIMESTAMP'], result['ID'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def load_all_from_db_desc_date_by_type(offset, limit, type=None, db=None):
        params = {'limit': limit, 'offset': offset}

        where_clause = ""

        if type is not None and type != "":
            params['type'] = type
            where_clause = "WHERE TYPE == :type "

        query = "SELECT * FROM 'EVENTS' " + where_clause + "ORDER BY TIMESTAMP DESC LIMIT :limit OFFSET :offset"

        return Event.load_all_from_db(query, params, db)

    @staticmethod
    def get_unseen_events(user, db):
        last_checked = user.events_last_checked

        events = Event.load_all_from_db("SELECT * FROM 'EVENTS' WHERE TIMESTAMP > :time",
                        {'time': last_checked}, db)

        return len(events)

    @staticmethod
    def load_all_types_from_db(types, db=None):
        return Event.load_all_from_db('SELECT * FROM EVENTS WHERE TYPE IN (%s)' % ','.join('?'*len(types)),
                                     types, db)

    @staticmethod
    #ID is username
    def load_from_db(id, db=None):
        items = Event.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (items[0].id != id)):
            raise ItemNotFoundException("Could not find event-id: " + id)
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return Event.load_all_from_db('SELECT * FROM EVENTS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            timestamp = dict['timestamp']
            text = dict['text']
            type = dict['type']

            user = Event(text, type, timestamp, id)

            return user
        except:
            raise InvalidParametersException("Event.create_from_dict(): invalid dict")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import urllib.error
import urllib.parse
import urllib.request

from homevee.Manager.api_key import get_api_key
from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_server_data, get_database_con


class WeatherUpdaterCronjob(IntervalCronjob):
    def __init__(self):
        super(WeatherUpdaterCronjob, self).__init__(task_name="WeatherUpdaterCronjob", interval_seconds=15*60)

    def task_to_do(self, *args):
        self.refresh_weather_cache()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (15 * 60) - ((t.minute * 60) - t.second) % 15 * 60

        return seconds_to_wait

    def refresh_weather_cache(self):
        db = get_database_con()

        api_key = get_api_key("Open Weather Map", db)
        location_id = get_server_data("WEATHER_LOCATION_ID", db)

        try:
            url = "http://api.openweathermap.org/data/2.5/forecast/daily?id=" + location_id + "&cnt=16&units=metric&lang=de&type=accurate&APPID=" + api_key
            response = urllib.request.urlopen(url).read()

            with db:
                cur = db.cursor()

                # Wetter-Daten in Datenbank schreiben
                cur.execute("INSERT OR REPLACE INTO SERVER_DATA (KEY, VALUE) values('WEATHER_CACHE', :response);",
                            {"response": response})
                db.commit()

            db.close()

            return True
        except:
            return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.rgb_control import rgb_control
from homevee.VoiceAssistant.Modules.DeviceControlModule import VoiceDeviceControlModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_RGB_CONTROL
from homevee.utils.colors import COLORS, COLOR_NAMES
from homevee.utils.device_types import *

class VoiceRgbDeviceControlModule(VoiceDeviceControlModule):
    def get_pattern(self, db):
        return PATTERN_RGB_CONTROL

    def get_label(self):
        return "rgb"

    def run_command(self, username, text, context, db):
        return self.voice_rgb_control(username, text, context, db)

    def voice_rgb_control(self, username, text, context, db):
        room = self.find_room(text, db)

        if room is not None:
            room_key = room['LOCATION']
        else:
            room_key = None

        device_types = [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]

        devices = self.find_devices(text, device_types, room_key, db)

        color = self.find_color(text)
        if color != False:
            color_hex = COLOR_NAMES[color]
        else:
            return "Du musst mir eine Farbe sagen."

        for device in devices:
            rgb_control(username, device['type'], device['id'], True, None, color_hex, db)

            return {'msg_speech': 'Ok.', 'msg_text': 'Ok.'}

    def find_color(self, text):
        for word in text.split():
            for color in COLORS:
                if word == color:
                    return color
        return False


#from BaseHTTPServer import BaseHTTPRequestHandler
from io import StringIO

'''
class HTTPRequest(BaseHTTPRequestHandler):
    def __init__(self, request_text):
        self.rfile = StringIO(request_text)
        self.raw_requestline = self.rfile.readline()
        self.error_code = self.error_message = None
        self.parse_request()

    def send_error(self, code, message):
        self.error_code = code
        self.error_message = message
'''


from homevee.Helper.helper_functions import has_permission
from homevee.items.Room import Room


def get_smart_speakers(username, db):
    speakers =[]

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM SMART_SPEAKER ORDER BY LOCATION ASC")

        for speaker in cur.fetchall():
            if has_permission(username, speaker['LOCATION'], db):
                item = {'name': speaker['NAME'], 'id': speaker['ID'], 'key': speaker['KEY'],
                        'location': speaker['LOCATION'], 'location_name': Room.get_name_by_id(speaker['LOCATION'],
                                                                                              db)}

                speakers.append(item)

    return {'speakers': speakers}

def add_edit_smart_speaker(username, db):
    return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from homevee.items.Status import *
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Gateway import *


def set_light_mode(username, id, mode, saturation, brightness, color, db, check_user=True):
    with db:
        cur = db.cursor()

        cur.execute("SELECT LOCATION FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", {'id': id})

        result = cur.fetchone()

        if result is None:
            return Status(type=STATUS_ERROR).get_dict()
        elif check_user and not has_permission(username, result['LOCATION'], db):
            return {'result': 'nopermission'}

        gateway = Gateway.load_from_db(PHILIPS_HUE_BRIDGE, db)

        if mode == True or mode == "1":
            mode = True
        else:
            mode = False

        data = {}

        cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", {'id': id})

        result = cur.fetchone()

        if color is not None:
            rgb = tuple(int(color[i:i + 2], 16) for i in (0, 2, 4))
            x, y, z = rgb_to_xy(rgb)
            data['xy'] = [x, y]
            data['sat'] = int(z*255)
        else:
            data['hue'] = result['HUE']

        if mode is not None:
            data['on'] = mode
        else:
            data['on'] = result['IS_ON'] == 1

        if brightness is not None:
            data['bri'] = int(float(brightness)*255/100)
        else:
            data['bri'] = int(result['BRIGHTNESS'])

        Logger.log("rgb")

        response = requests.put("http://" + gateway.ip + "/api/" + gateway.key1 + "/lights/" + str(id) + "/state", data=json.dumps(data))

        Logger.log(response.text)

        update_light_info(id, mode, color, saturation, brightness, db)

        return Status(type=STATUS_OK).get_dict()

def rgb_to_xy(rgb):
    red, green, blue = rgb

    if( red > 0.04045):
        red = pow((red + 0.055) / (1.0 + 0.055), 2.4)
    else:
        red = (red / 12.92)

    if(green > 0.04045):
        green = pow((green + 0.055) / (1.0 + 0.055), 2.4)
    else: green = (green / 12.92)

    if(blue > 0.04045):
        blue = pow((blue + 0.055) / (1.0 + 0.055), 2.4)
    else:
        blue = (blue / 12.92)

    X = red * 0.664511 + green * 0.154324 + blue * 0.162028
    Y = red * 0.283881 + green * 0.668433 + blue * 0.047685
    Z = red * 0.000088 + green * 0.072310 + blue * 0.986039

    if (X+Y+Z) == 0:
        fx = 0
        fy = 0
        fz = 0
    else:
        fx = X / (X + Y + Z)
        fy = Y / (X + Y + Z)
        fz = Z / (X + Y + Z)

    if (fx != fx):
        fx = 0.0

    if (fy != fy):
        fy = 0.0

    if (fz != fz):
        fz = 0.0

    return (fx, fy, fz)

def get_devices(db):
    gateway = Gateway.load_from_db(PHILIPS_HUE_BRIDGE, db)

    try:
        response = requests.get("http://" + gateway.ip + "/api/"+gateway.key1+"/lights").text

        Logger.log(response)

        data = json.loads(response)

        ids = list(data.keys())

        devices = []

        for id in ids:
            item = data[id]
            device_item = {'title': item['name'], 'id': id, 'room': None, 'type': 'light'}
            devices.append(device_item)
    except:
        Logger.log("Could not connect to Hue-Bridge...")

    return {'devices': devices}

def get_light_info(id, db):
    gateway = Gateway.load_from_db(PHILIPS_HUE_BRIDGE, db)

    try:
        request = requests.get("http://" + gateway.ip + "/api/" + gateway.key1 + "/lights/"+str(id))

        code = request.status_code

        if(code != 200):
            return False

        data = json.loads(request.text)

        mode = data['state']['on']
        brightness = int(float(data['state']['bri'])/255*100)
        saturation = int(float(data['state']['sat'])/255*100)
        hue = data['state']['hue']

        update_light_info(id, mode, hue, saturation, brightness, db)

        return True
    except:
        Logger.log("Could not connect to Hue-Bridge...")
        return False

def update_light_info(id, mode, hue, saturation, brightness, db):
    with db:
        cur = db.cursor()

        if saturation is None:
            saturation = 255

        if brightness is None:
            brightness = 255

        cur.execute("UPDATE PHILIPS_HUE_LIGHTS SET IS_ON = :mode, BRIGHTNESS = :bri, SATURATION = :sat, HUE = :hue WHERE ID = :id",
                    {'mode': mode, 'bri': brightness, 'sat': saturation, 'hue': hue, 'id': id})


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.items.Status import *

def add_edit_person(username, id, name, nickname, address, latitude, longitude, phonenumber, birthdate, db):
    id = int(id)

    with db:
        cur = db.cursor()

        params = {'name': name, 'nickname': nickname, 'address': address, 'lat': latitude, 'lng': longitude,
                  'birthdate': birthdate, 'number': phonenumber}

        if id is None or id == -1:
            cur.execute("INSERT INTO PEOPLE_DATA (NAME, NICKNAME, ADDRESS, LATITUDE, LONGITUDE, BIRTHDATE, PHONE_NUMBER) VALUES (:name, :nickname, :address, :lat, :lng, :birthdate, :number)",
                        params)
        else:
            params['id'] = id

            cur.execute('UPDATE PEOPLE_DATA SET NAME = :name, NICKNAME = :nickname, ADDRESS = :address, LATITUDE = :lat, LONGITUDE = :lng, BIRTHDATE = :birthdate, PHONE_NUMBER = :number WHERE ID = :id',
                        params)

    return Status(type=STATUS_OK).get_dict()

def get_persons(db):
    persons = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PEOPLE_DATA")

        for person in cur.fetchall():
            item = {'id': person['ID'], 'name': person['NAME'], 'nickname': person['NICKNAME'], 'address': person['ADDRESS'],
                    'latitude': person['LATITUDE'], 'longitude': person['LONGITUDE'], 'phonenumber': person['PHONE_NUMBER'],
                    'birthdate': person['BIRTHDATE']}
            persons.append(item)

    return {'persons': persons}


import json

from pymax.cube import Cube

from homevee.items.Gateway import Gateway


class MaxCube(Gateway):
    def __init__(self, gateway):
        super(MaxCube, self).__init__(gateway.name, gateway.ip,
            gateway.port, gateway.key1, gateway.key2, gateway.type)

    def get_devices(self):
        data = []

        with Cube(self.ip) as cube:
            for room in cube.rooms:
                for device in room.devices:
                    devicedata = {}

                    devicedata['room'] = room.name
                    devicedata['title'] = device.name
                    # devicedata['temp'] = device.temperature
                    devicedata['id'] = device.serial

                    data.append(devicedata)

        return json.dumps(data)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import codecs
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request

from homevee.Helper import translations, Logger
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.helper import generate_string

GREETINGS = ['hallo', 'hey', 'na', 'hi']

THANKS = ['danke', 'dank', 'dankeschön']

class VoiceConversationModule(VoiceModule):
    def get_context_key(self):
        return "VOICE_CONVERSATION"

    def get_priority(self):
        return 0

    def get_pattern(self, db):
        return []

    def get_label(self):
        return "conversation"

    def run_command(self, username, text, context, db):
        Logger.log("conversation: "+text)
        return self.conversation(username, text, context, db)

    def conversation(self, username, text, context, db):
        url = helper.SMART_API_PATH + "/?action=plaintext&language="+translations.LANGUAGE+"&text="+urllib.parse.quote(codecs.encode(text, 'utf-8'))
        Logger.log("calling: "+url)
        output = urllib.request.urlopen(url).read()

        #print(url)

        #print(output)

        if output is None or output == '' or len(output)==0:
            data = [
                [[['Es ', 'tut '], 'Tut '], 'mir ', ['echt ', 'sehr ', ''], ['Leid. ', 'Leid, '+username+'. '],
                ['Da ', 'Dabei ', 'Damit '], 'kann ', 'ich ', 'dir ', ['leider ', ''], ['noch ', ''], 'nicht ', ['helfen', 'weiterhelfen'], '.']
            ]
            output = generate_string(data)
        else:
            output = output.decode("utf-8")

        result = {'msg_speech': output, 'msg_text': output}

        return result

    def contains(self, text, array):
        words = text.split(' ')

        for item in array:
            if item in words:
                return True

        return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import io
import json
import os
import time
import traceback
from _thread import start_new_thread

from PIL import Image

from homevee.Functions.tensorflow_functions import people_predict, people_retrain
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *
from homevee.utils import constants
from homevee.utils.constants import DATA_DIR
from homevee.utils.database import get_database_con, set_server_data, get_server_data
from homevee.utils.file_utils import create_image

IS_TRAINING_TAG = "PEOPLE_CLASSIFIER_TRAINING_RUNNING"

def upload_images(username, data, img_class, db):
    images = json.loads(data)

    counter = 0

    if(img_class == "-1" or img_class == -1):
        img_class = None

    for image in images:
        filename = "person-" + str(int(time.time())) + "_" + str(counter)

        counter += 1

        image_path = create_image(filename, "people", image, optimize=True)

        with db:
            cur = db.cursor()

            cur.execute("INSERT INTO PEOPLE_LEARNING_DATA (PATH, PERSON_ID) VALUES (:path, :person_id)",
                        {'path': image_path, 'person_id': img_class})

        #print image

    return Status(type=STATUS_OK).get_dict()

def save_people_class(username, id, data, classname, db):
    return

def classify_person(image_data, db):

    filename = "person-"+str(int(time.time()))

    image_path = create_image(filename, "people", image_data, optimize=True)

    with db:
        cur = db.cursor()

        cur.execute("INSERT INTO PEOPLE_LEARNING_DATA (PATH) VALUES (:path)",
                    {'path': image_path})

    predictions = people_predict.predict(os.path.join(constants.DATA_DIR, image_path))

    for prediction in predictions:
        person_data = get_person_class_data(prediction['prediction'], db)
        Logger.log(person_data['NAME'] + ": " + str(prediction['confidence']))

    Logger.log(image_path)

    person_data = get_person_class_data(predictions[0]['prediction'], db)

    return {'label': person_data['NAME'], 'confidence': str(predictions[0]['confidence']*100)}

def get_person_class_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PEOPLE_DATA WHERE ID = :id;", {'id': id})

        return cur.fetchone()

def start_people_training(username, db):
    if not has_permission(username, 'admin', db):
        return {'result': 'noadmin'}

    is_training = get_server_data(IS_TRAINING_TAG, db)

    if(is_training is not None and is_training == "true"):
        return {'result': 'alreadytraining'}

    start_new_thread(training_thread, (username, None))

    return Status(type=STATUS_OK).get_dict()

def training_thread(username, d):
    db = get_database_con()

    set_server_data(IS_TRAINING_TAG, "true", db)

    try:
        people_retrain.people_training(username)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

    set_server_data(IS_TRAINING_TAG, "false", db)

def get_people_classes(db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PEOPLE_DATA")

        classes = []

        for row in cur.fetchall():
            item = {'id': int(row['ID']), 'name': row['NAME'],}
            classes.append(item)

        item = {'id': None, 'name': 'Nicht zugeordnet'}
        classes.append(item)

        return {'classes': classes}

def get_people_class_images(image_class, show, offset, db):
    param_array = {'limit': show, 'offset': offset}

    with db:
        cur = db.cursor()

        if image_class is None or image_class == "-1" or image_class == -1:
            cur.execute("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID IS NULL OR PERSON_ID = -1 LIMIT :limit OFFSET :offset",
                        param_array)
        else:
            param_array['key'] = image_class
            cur.execute("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID = :key LIMIT :limit OFFSET :offset",
                        param_array)

        images = []

        for img in cur.fetchall():
            images.append(img['ID'])

        return {'images': images}

def delete_people_images(ids, db):
    # todo

    ids = json.loads(ids)

    with db:
        cur = db.cursor()

        for id in ids:
            Logger.log("deleting: "+str(id))
            cur.execute("SELECT * FROM PEOPLE_LEARNING_DATA WHERE ID == :id;",
                        {'id': id})
            item = cur.fetchone()
            if (item is not None):
                rel_path = item['PATH']

                image_path = os.path.join(DATA_DIR, rel_path)
                if os.path.exists(image_path):
                    os.remove(image_path)
                else:
                    Logger.log(image_path+" not found")

                cur.execute("DELETE FROM PEOPLE_LEARNING_DATA WHERE ID == :id;",
                            {'id': id})

    return Status(type=STATUS_OK).get_dict()

def change_people_image_class(ids, newclass, db):
    if newclass == "-1":
        newclass = None

    ids = json.loads(ids)

    Logger.log(ids)

    with db:
        cur = db.cursor()

        for id in ids:
            cur.execute("UPDATE 'PEOPLE_LEARNING_DATA' SET PERSON_ID = :key WHERE ID == :id;",
                        {'key': newclass, 'id': id})

    return Status(type=STATUS_OK).get_dict()

def get_people_images(image_class, show, offset, db):
    param_array = {'limit': show, 'offset': offset}

    with db:
        cur = db.cursor()

        if image_class == "-1":
            cur.execute("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID IS NULL LIMIT :limit OFFSET :offset",
                        param_array)
        else:
            param_array['key'] = image_class
            cur.execute("SELECT * FROM 'PEOPLE_LEARNING_DATA' WHERE PERSON_ID = :key LIMIT :limit OFFSET :offset",
                        param_array)

        images = []

        for img in cur.fetchall():
            images.append(img['ID'])

        return {'images': images}

def get_people_image(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT PATH FROM 'PEOPLE_LEARNING_DATA' WHERE ID == :id",
                    {'id': id})

        rel_path = cur.fetchone()['PATH']

        path = os.path.join(DATA_DIR, rel_path)
        Logger.log(("Path: " + path))

        im = Image.open(path)

        size = 50

        image_dimensions = size, size

        im.thumbnail(image_dimensions, Image.ANTIALIAS)

        buffer = io.BytesIO()
        im.save(buffer, format="JPEG")
        encoded_string = base64.b64encode(buffer.getvalue())
        encoded_string = encoded_string.decode('utf-8')
        im.close()

        #with open(path, "rb") as image_file:
        #    encoded_string = base64.b64encode(image_file.read())

    return {'imagedata': encoded_string}

def get_performance_settings(username, db):
    if not has_permission(username, 'admin', db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    prefix = "PEOPLE_"

    tf_data = {}

    with db:
        cur = db.cursor()

        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

        for index in indices:
            cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                        {'key': prefix+index})

            result = cur.fetchone()
            if(result is not None and 'VALUE' in result):
                tf_data[index] = result['VALUE']

    return {'data': tf_data, 'graph_sizes': people_retrain.get_graph_sizes(),
            'image_sizes' : people_retrain.get_image_sizes()}

def set_performance_settings(username, data, db):
    if not has_permission(username, 'admin', db):
        return {'result': 'noadmin'}

    prefix = "PEOPLE_"

    data = json.loads(data)

    with db:
        cur = db.cursor()

        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

        for index in indices:
            param_array = {'value': data[index], 'key': prefix+index}

            cur.execute("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key;", param_array)

            cur.execute("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES (:value, :key);", param_array)

    return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.Functions.tv_data import get_tv_plan
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_TV_TIPPS

class VoiceGetTvTippsModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_TV_TIPPS

    def get_label(self):
        return "gettvtipps"

    def run_command(self, username, text, context, db):
        return self.get_tv_tipps(username, text, context, db)

    def get_tv_tipps(self, username, text, context, db):
        tv_shows = get_tv_plan(username, "tipps", db)

        VERB = ['kommt', 'läuft']

        show_strings = []
        for show in tv_shows:
            TIME = show['time'] + ' Uhr'
            CHANNEL = show['channel']
            NAME = '\''+show['name']+'\''

            string_data = [
                ['um ', TIME, ' ', VERB, ' auf ', CHANNEL,  ' ', NAME],
                ['um ', TIME, ' ', VERB, ' ', NAME, ' auf ', CHANNEL],
                ['auf ', CHANNEL, ' um ', TIME, ' ', VERB, ' ', NAME],
                ['auf ', CHANNEL, ' ', VERB, ' ', NAME, ' um ', TIME],
                [NAME, ' ', VERB, ' um ', TIME, ' auf ', CHANNEL],
                [NAME, ' ', VERB, ' auf ', CHANNEL, ' um ', TIME],
            ]

            show_strings.append(helper.generate_string(string_data))

        show_string = None
        for i in range(0, len(show_strings)):
            if show_string is None:
                show_string = show_strings[i]
            elif i == len(show_strings)-1:
                if i != 0:
                    show_string = show_string + ' und '
                show_string = show_string+show_strings[i]
            else:
                show_string = show_string+', '+show_strings[i]

        answer_data = [
            [['Die', 'Deine'], ' ', ['TV', 'Fernseh', 'Programm'], '-', ['Tipps', 'Vorschläge', 'Empfehlungen'], ' ',
                ['heute', 'für heute'], ' ', ['sind', 'lauten'], ':']
        ]

        output = helper.generate_string(answer_data) + ' ' + show_string + '.'

        return {'msg_speech': output, 'msg_text': output}


from homevee.exceptions import ItemNotFoundException
from homevee.items import Item
from homevee.utils import database


class ShoppingListItem(Item):
    def __init__(self, item, amount, id=None):
        super(ShoppingListItem, self).__init__()

        self.id = id
        self.item = item
        self.amount = amount

    def delete(self, db=None):
        try:
            if (db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM SHOPPING_LIST WHERE ID == :id", {'id': self.id})
                return True
        except:
            return False

    def save_to_db(self, db=None):
        if (db is None):
            db = database.get_database_con()

        with db:
            cur = db.cursor()

            if(self.id is None or self.id == ""):
                #TODO Check if entry with same name exists and notify user(???)

                cur.execute("INSERT OR IGNORE INTO SHOPPING_LIST (AMOUNT, ITEM) VALUES (:amount, :name);",
                            {'amount': self.amount, 'name': self.name})
            else:
                cur.execute("UPDATE OR IGNORE SHOPPING_LIST SET AMOUNT = :amount, ITEM = :name WHERE ID = :id",
                            {'amount': self.amount, 'name': self.name, 'id': self.id})

    def build_dict(self):
        dict = {
            'id': self.id,
            'item': self.item,
            'amount': self.amount
        }
        return dict

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return ShoppingListItem.load_all_from_db('SELECT * FROM SHOPPING_LIST WHERE ID IN (%s)' % ','.join('?' * len(ids)),
                                     ids, db)

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if(db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()
            cur.execute(query, params)

            for item in cur.fetchall():
                if item['AMOUNT'] is None:
                    amount = -1
                else:
                    amount = int(item['AMOUNT'])

                items.append(ShoppingListItem(item['ITEM'], amount, item['ID']))

            cur.close()

        return items

    @staticmethod
    def load_all(db=None):
        return ShoppingListItem.load_all_from_db('SELECT * FROM SHOPPING_LIST', {}, db)

    @staticmethod
    def load_from_db(id, db=None):
        items = ShoppingListItem.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find shopping-list-item with id: " + id)
        else:
            return items[0]

    @staticmethod
    def create_from_dict(dict):
        id = dict['id']
        item = dict['item']
        amount = dict['amount']

        return ShoppingListItem(item, amount, id)


#!/usr/bin/python
# -*- coding: utf-8 -*-

import time

from homevee.items.Event import Event

'''Gibt die Ereignisse zurück'''
def get_events(user, type, limit, offset, db):
    events = Event.load_all_from_db_desc_date_by_type(offset, limit, type)

    user.events_last_checked = time.time()
    user.save_to_db(db)

    return {'events': Event.list_to_dict(events)}

'''Gibt die Anzahl der ungesehenen Ereignisse zurück'''
def get_unseen_events(user, db):
    count = Event.get_unseen_events(user, db)

    return {'eventcount': count}

'''Gibt die vorhandenen Ereignis-Typen zurück'''
def get_event_types(db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT DISTINCT TYPE FROM EVENTS ORDER BY TYPE")

        types = []

        for data in cur.fetchall():
            types.append(data['TYPE'])

        return {'types': types}

'''Erstellt ein neues Ereignis'''
def add_event(type, text, db):
    event = Event(text=text, type=type)
    return event.save_to_db()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os

from homevee.DeviceAPI import max_cube, zwave, philips_hue, rademacher_homepilot
from homevee.Functions.room_data import get_rooms
from homevee.Functions.sensor_data import SENSOR_TYPE_MAP
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Gateway import *
from homevee.items.Status import *
from homevee.utils.device_types import *


def get_gateway_devices(username, type, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    if type == Z_WAVE_GATEWAY:
        return zwave.get_devices.get_devices(db)
    if type == PHILIPS_HUE_BRIDGE:
        return philips_hue.get_devices(db)
    if type == RADEMACHER_HOMEPILOT:
        return rademacher_homepilot.get_devices(db)
    elif type == MAX_CUBE:
        gateway = Gateway.load_from_db("MAX! Cube", db)

        Logger.log(gateway.ip)

        devices = max_cube.get_devices(gateway.ip)

        if devices is not None:
            return {'devices': devices}
        else:
            return {'devices': []}
    elif type == "Network":
        network_devices = []

        hostnames = os.popen("arp -a").read()

        ip_adresses = os.popen("arp -a").read()

        Logger.log(hostnames)
        #print ip_adresses

        #for i in range(0, len(hostnames)):

        return {'devices': network_devices}
    else:
        return "nosuchgateway"

'''
function getGatewayDevices($type, $db){
	switch($type){
		case "Network":
			$networkDevices = array();

			exec("arp", $hostnames);
			exec("arp -n", $ipAdresses);

			for($i = 1; $i < sizeOf($hostnames); $i++){

				$names = preg_replace('!\s+!', ' ', $hostnames[$i]);

				$name = explode(" ", $names)[0];

				$adresses = preg_replace('!\s+!', ' ', $ipAdresses[$i]);

				$data = explode(" ", $adresses);

				$ip = $data[0];

				$mac = $data[2];

				array_push($networkDevices, array('id' => $ip, 'title' => $name, 'id2' => $mac));
			}

			return array('devices' => $networkDevices);
		default:
			return "nosuchgateway";
	}
}
'''

def add_edit_device(username, device_id, type, data, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    add_new = (device_id == None or device_id == "" or device_id == "-1")

    type = type.decode("utf-8")

    data_array = json.loads(data)

    with db:
        cur = db.cursor()

        #XBOX One WOL
        if type == XBOX_ONE_WOL:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                           'ip': data_array['ip'], 'live_id': data_array['liveid']}

            if device_id == "-1" or device_id is -1:
                cur.execute(
                    "INSERT OR IGNORE INTO 'XBOX_ONE_WOL' (LOCATION, ICON, NAME, IP_ADDRESS, XBOX_LIVE_ID) VALUES (:room, :icon, :name, :ip, :live_id);",
                    param_array)
            else:

                param_array['id'] = device_id

                cur.execute(
                    "UPDATE OR IGNORE 'XBOX_ONE_WOL' SET LOCATION = :room, ICON = :icon, NAME = :name, IP_ADDRESS = :ip, XBOX_LIVE_ID = :live_id WHERE ID == :id;",
                    param_array)

        # Philips Hue
        elif type == PHILIPS_HUE_LIGHT:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

            cur.execute(
                "INSERT OR IGNORE INTO 'PHILIPS_HUE_LIGHTS' (LOCATION, ID, ICON, NAME) VALUES (:room, :id, :icon, :name);",
                param_array)

            cur.execute(
                "UPDATE OR IGNORE 'PHILIPS_HUE_LIGHTS' SET LOCATION = :room, ICON = :icon, NAME = :name WHERE ID == :id;",
                param_array)
        #MQTT Sensor
        elif type == MQTT_SENSOR:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data['icon'], 'id': device_id,
                           'key': data_array['key'], 'topic': data_array['topic'], 'save_data': data_array['save_data']}

            cur.execute(
                "INSERT OR IGNORE INTO 'MQTT_SENSORS' (ROOM, ID, ICON, NAME, KEY, TOPIC, SAVE_DATA) VALUES (:room, :id, :icon, :name, :key, :topic, :save_data);",
                param_array)

            cur.execute(
                "UPDATE OR IGNORE 'MQTT_SENSORS' SET ROOM = :room, ICON = :icon, NAME = :name, KEY = :key, TOPIC = :topic, SAVE_DATA = :save_data, WHERE ID == :id;",
                param_array)

        #433 MHz
        elif type == FUNKSTECKDOSE:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'],
                           'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']}

            '''if add_new:
                            cur.execute("SELECT COUNT(*) FROM 'funksteckdosen' WHERE HAUSCODE = :hauscode AND STECKDOSENNUMMER = :nummer;",
                                        {'hauscode': data_array['hauscode'], 'nummer': data_array['steckdosennummer']})

                            count = cur.fetchone()['COUNT(*)']

                            if(int(count) != 0):
                                return "hauscodesocketnumberinuse"
                        else:
                            param_array['id'] = device_id'''

            if add_new:
                cur.execute("INSERT INTO 'funksteckdosen' (ROOM, HAUSCODE, STECKDOSENNUMMER, ICON, NAME) VALUES (:room, :hauscode, :nummer, :icon, :name);",
                            param_array)
            else:
                param_array['id'] = device_id

                cur.execute("UPDATE 'funksteckdosen' SET ROOM = :room, HAUSCODE = :hauscode, STECKDOSENNUMMER = :nummer, ICON = :icon, NAME = :name WHERE DEVICE == :id;",
                            param_array)
        # MAX!
        elif type == MAX_THERMOSTAT:
            param_array = {'room': data_array['room'], 'name': data_array['name'],
                           'icon': data_array['icon'], 'id': device_id}

            cur.execute("UPDATE OR IGNORE 'MAX_THERMOSTATS' SET RAUM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                param_array)

            cur.execute("INSERT OR IGNORE INTO 'MAX_THERMOSTATS' (RAUM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                param_array)
        # Rademacher!
        elif type == RADEMACHER_THERMOSTAT:
            param_array = {'room': data_array['room'], 'name': data_array['name'],
                           'icon': data_array['icon'], 'id': device_id}

            cur.execute("UPDATE OR IGNORE 'HOMEPILOT_THERMOSTATS' SET ROOM = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                param_array)

            cur.execute("INSERT OR IGNORE INTO 'HOMEPILOT_THERMOSTATS' (ROOM, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                param_array)
        elif type == RADEMACHER_BLIND_CONTROL:
            param_array = {'room': data_array['room'], 'name': data_array['name'],
                           'icon': data_array['icon'], 'id': device_id}

            cur.execute("UPDATE OR IGNORE 'HOMEPILOT_BLIND_CONTROL' SET LOCATION = :room, NAME = :name, ICON = :icon WHERE ID = :id;",
                param_array)

            cur.execute("INSERT OR IGNORE INTO 'HOMEPILOT_BLIND_CONTROL' (LOCATION, ID, NAME, ICON) VALUES (:room, :id, :name, :icon);",
                param_array)

        #Z-Wave
        elif type == ZWAVE_SENSOR:
            param_array = {'room': data_array['room'], 'savedata': data_array['savedata'], 'shortform': data_array['name'],
                           'sensor_type': data_array['sensor_type'], 'icon': data_array['icon'], 'id': device_id}

            cur.execute("SELECT COUNT(*) FROM ZWAVE_SENSOREN WHERE ID = :id", {'id': device_id})
            add_new = (cur.fetchone()['COUNT(*)'] == 0)

            if add_new:
                cur.execute("INSERT INTO 'ZWAVE_SENSOREN' (RAUM, ID, SHORTFORM, ICON, SAVE_DATA, SENSOR_TYPE) VALUES (:room, :id, :shortform, :icon, :savedata, :sensor_type);",
                            param_array)
            else:
                cur.execute("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :room, SHORTFORM = :shortform, ICON = :icon, SAVE_DATA = :savedata, SENSOR_TYPE = :sensor_type WHERE ID = :id;",
                            param_array)
        elif type == ZWAVE_THERMOSTAT:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

            cur.execute("SELECT COUNT(*) FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", {'id': device_id})
            add_new = (cur.fetchone()['COUNT(*)'] == 0)

            if add_new:
                cur.execute("INSERT INTO 'ZWAVE_THERMOSTATS' (RAUM, NAME, THERMOSTAT_ID, ICON) VALUES (:room,:name,:id,:icon);",
                param_array)
            else:
                cur.execute("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :room, NAME = :name, THERMOSTAT_ID = :id, ICON =:icon WHERE THERMOSTAT_ID = :id;",
                param_array)
        elif type == ZWAVE_POWER_METER:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id, 'daily_reset': data_array['is_reset_daily']}

            cur.execute("SELECT COUNT(*) FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': device_id})
            add_new = (cur.fetchone()['COUNT(*)'] == 0)

            if add_new:
                cur.execute("INSERT INTO 'ZWAVE_POWER_METER' (ROOM_ID, DEVICE_ID, DEVICE_NAME, ICON, IS_RESET_DAILY) VALUES (:room,:id,:name,:icon,:daily_reset);",
                param_array)
            else:
                cur.execute("UPDATE 'ZWAVE_POWER_METER' SET ROOM_ID = :room, DEVICE_NAME = :name, DEVICE_ID = :id, ICON =:icon, IS_RESET_DAILY = :daily_reset WHERE DEVICE_ID =:id;",
                param_array)
        elif type == ZWAVE_SWITCH:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

            cur.execute("SELECT COUNT(*) FROM ZWAVE_SWITCHES WHERE ID = :id", {'id': device_id})
            add_new = (cur.fetchone()['COUNT(*)'] == 0)

            if add_new:
                cur.execute("INSERT INTO 'ZWAVE_SWITCHES' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
                param_array)
            else:
                cur.execute("UPDATE 'ZWAVE_SWITCHES' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
                param_array)
        elif type == ZWAVE_DIMMER:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'icon': data_array['icon'], 'id': device_id}

            cur.execute("SELECT COUNT(*) FROM ZWAVE_DIMMER WHERE ID = :id", {'id': device_id})
            add_new = (cur.fetchone()['COUNT(*)'] == 0)

            if add_new:
                cur.execute("INSERT INTO 'ZWAVE_DIMMER' (LOCATION, NAME, ID, ICON) VALUES (:room,:name,:id,:icon);",
                param_array)
            else:
                cur.execute("UPDATE 'ZWAVE_DIMMER' SET LOCATION = :room, NAME = :name, ID = :id, ICON =:icon WHERE ID =:id;",
                param_array)
        #Media Center
        elif type == MEDIA_CENTER:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'ip': data_array['ip'],
                           'username': data_array['username'], 'password': data_array['password'], 'type': data_array['type']}

            if add_new:
                cur.execute("INSERT INTO 'MEDIA_CENTER' (LOCATION, NAME, IP, USERNAME, PASSWORD, TYPE) VALUES (:room,:name, :ip, :username, :password, :type);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'MEDIA_CENTER' SET LOCATION = :room, NAME = :name, IP = :id, USERNAME = :username, PASSWORD = :password, TYPE = :type, ICON =:icon WHERE ID = :id;",
                param_array)
        #URL-Devices
        elif type == URL_SWITCH:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'on': data_array['onurl'],
                           'off': data_array['offurl'], 'val': data_array['valurl'], 'icon': data_array['icon']}

            if add_new:
                cur.execute("INSERT INTO 'URL_SWITCH_BINARY' (LOCATION, NAME, URL_ON, URL_OFF, URL_GET_STATE, ICON) VALUES (:room,:name, :on, :off, :val, :icon);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'URL_SWITCH_BINARY' SET LOCATION = :room, NAME = :name, URL_ON = :on, URL_OFF = :off, URL_GET_STATE = :val, ICON =:icon WHERE ID = :id;",
                param_array)
        elif type == URL_TOGGLE:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'toggle': data_array['toggleurl'],
                           'icon': data_array['icon']}

            if add_new:
                cur.execute("INSERT INTO 'URL_TOGGLE' (LOCATION, NAME, TOGGLE_URL, ICON) VALUES (:room, :name, :toggle, :icon);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'URL_TOGGLE' SET LOCATION = :room, NAME = :name, TOGGLE_URL = :toggle, ICON =:icon WHERE ID = :id;",
                param_array)
        elif type == URL_RGB_LIGHT:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'url': data_array['url'],
                           'icon': data_array['icon']}

            if add_new:
                cur.execute("INSERT INTO 'URL_RGB_LIGHT' (LOCATION, NAME, URL, ICON) VALUES (:room, :name, :url, :icon);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'URL_RGB_LIGHT' SET LOCATION = :room, NAME = :name, URL = :toggle, ICON =:icon WHERE ID = :id;",
                param_array)
        #MQTT
        elif type == MQTT_TRIGGER:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                           'icon': data_array['icon'], 'type': data_array['type']}

            if add_new:
                cur.execute("INSERT INTO 'MQTT_TRIGGERS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'MQTT_TRIGGERS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
                param_array)
        elif type == MQTT_SENSOR:
            param_array = {'room': data_array['room'], 'name': data_array['name'], 'topic': data_array['topic'],
                           'icon': data_array['icon'], 'type': data_array['type']}

            if add_new:
                cur.execute("INSERT INTO 'MQTT_SENSORS' (LOCATION, NAME, TOPIC, TYPE, ICON) VALUES (:room, :name, :topic, :type, :icon);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'MQTT_SENSORS' SET LOCATION = :room, NAME = :name, TOPIC = :topic, ICON =:icon WHERE ID = :id;",
                param_array)
        #Wake on Lan
        elif type == WAKE_ON_LAN:
            param_array = {'room': data_array['room'], 'name': data_array['name'],
                           'icon': data_array['icon'], 'mac': data_array['mac']}

            if add_new:
                cur.execute("INSERT INTO 'WAKE_ON_LAN' (LOCATION, NAME, ICON, MAC_ADDRESS) VALUES (:room, :name, :icon, :mac);",
                param_array)
            else:
                param_array['id'] = device_id
                cur.execute("UPDATE 'WAKE_ON_LAN' SET LOCATION = :room, NAME = :name, MAC_ADDRESS = :mac, ICON =:icon WHERE DEVICE = :id;",
                param_array)
        else:
            return {'result': 'nosuchtype'}

    return Status(type=STATUS_OK).get_dict()

def delete_device(username, type, id, db):
    #type = type.encode('utf-8')

    with db:
        cur = db.cursor()

        db_cols = {
            FUNKSTECKDOSE: {'table': 'FUNKSTECKDOSEN', 'location': 'ROOM', 'id': 'DEVICE'},
            PHILIPS_HUE_LIGHT: {'table': 'PHILIPS_HUE_LIGHTS', 'location': 'LOCATION', 'id': 'ID'},
            WAKE_ON_LAN: {'table': 'WAKE_ON_LAN', 'location': 'LOCATION', 'id': 'DEVICE'},
            XBOX_ONE_WOL: {'table': 'XBOX_ONE_WOL', 'location': 'LOCATION', 'id': 'ID'},
            URL_RGB_LIGHT: {'table': 'URL_RGB_LIGHT', 'location': 'LOCATION', 'id': 'ID'},
            MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'location': 'ROOM', 'id': 'ID'},
            URL_TOGGLE: {'table': 'URL_TOGGLE', 'location': 'LOCATION', 'id': 'ID'},
            URL_SWITCH: {'table': 'URL_SWITCH', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_DIMMER: {'table': 'ZWAVE_DIMMER', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_SWITCH: {'table': 'ZWAVE_SWITCHES', 'location': 'LOCATION', 'id': 'ID'},
            ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'location': 'RAUM', 'id': 'ID'},
            ZWAVE_POWER_METER: {'table': 'ZWAVE_POWER_METER', 'location': 'ROOM_ID', 'id': 'DEVICE_ID'},
            ZWAVE_THERMOSTAT: {'table': 'ZWAVE_THERMOSTATS', 'location': 'RAUM', 'id': 'THERMOSTAT_ID'},
            MAX_THERMOSTAT: {'table': 'MAX_THERMOSTATS', 'location': 'RAUM', 'id': 'ID'},
            RADEMACHER_THERMOSTAT: {'table': 'HOMEPILOT_THERMOSTATS', 'location': 'ROOM', 'id': 'ID'},
            RADEMACHER_BLIND_CONTROL: {'table': 'HOMEPILOT_BLIND_CONTROL', 'location': 'LOCATION', 'id': 'ID'}
        }

        if type not in db_cols:
            return {'result': 'nosuchtype'}

        cur.execute("SELECT * FROM "+db_cols[type]['table']+" WHERE "+db_cols[type]['id']+" = :id",
                    {'id': id})

        data = cur.fetchone()

        Logger.log(data)

        if not has_permission(username, data[db_cols[type]['location']], db):
            return {'result': 'nopermission'}

        cur.execute("DELETE FROM "+db_cols[type]['table']+" WHERE " + db_cols[type]['id'] + " = :id",
                    {'id': id})

        return Status(type=STATUS_OK).get_dict()

'''
function addEditDevice($deviceId, $type, $data, $db){
	if(!hasPermission("admin", $db)){
		return "noadmin";
	}
	
	$addNew = ($deviceId == null || $deviceId == "" || $deviceId == "-1");
	
	$type = utf8_decode($type);
	
	$dataArray = json_decode($data, true);
	
	switch($type){
		//Z-Wave
		case "Z-Wave Dimmer":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "icon" => $dataArray['icon']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'ZWAVE_DIMMER' (LOCATION, NAME, ID, ICON)
									VALUES (:room, :name, :id, :icon);");
			}	
			else{
				$query = $db->prepare("UPDATE 'ZWAVE_DIMMER' SET LOCATION = :room, NAME = :name, ID = :id,
									ICON = :icon WHERE ID = :id;");
			}
			
			//Z-Wave Dimmer bearbeiten bzw. hinzufügen
			break;
		//DIY
		case "DIY Fensterkontakt":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "ip" => $dataArray['ip'],
			"icon" => $dataArray['icon'], "channel" => $dataArray['channel']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'DIY_REEDSENSORS' (RAUM, ICON, IP, NAME, CHANNEL)
									VALUES (:room, :icon, :ip, :name, :channel);");
			}	
			else{
				$query = $db->prepare("UPDATE 'DIY_REEDSENSORS' SET RAUM = :room, ICON = :icon, IP = :ip, NAME = :name, 
					CHANNEL = :channel WHERE ID = :id;");
			}
			
			//DIY Sensor bearbeiten bzw. hinzufügen
			break;
		case "DIY Sensor":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "ip" => $dataArray['ip'], "savedata" => $dataArray['savedata'],
				"einheit" => $dataArray['einheit'], "icon" => $dataArray['icon'], "channel" => $dataArray['channel']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'DIY_SENSORS' (RAUM, ICON, IP, NAME, EINHEIT, SAVE_DATA, CHANNEL)
									VALUES (:room, :icon, :ip, :name, :einheit, :savedata, :channel);");
			}	
			else{
				$query = $db->prepare("UPDATE 'DIY_SENSORS' SET RAUM = :room, ICON = :icon, IP = :ip, NAME = :name, EINHEIT = :einheit,
									SAVE_DATA = :savedata, CHANNEL = :channel WHERE ID = :id;");
			}
			
			//DIY Sensor bearbeiten bzw. hinzufügen
			break;
		case "DIY Anwesenheitssensor":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "ip" => $dataArray['ip'],
			"icon" => $dataArray['icon'], "channel" => $dataArray['channel']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'DIY_PRESENCESENSORS' (RAUM, ICON, IP, NAME, CHANNEL)
									VALUES (:room, :icon, :ip, :name, :channel);");
			}	
			else{
				$query = $db->prepare("UPDATE 'DIY_PRESENCESENSORS' SET RAUM = :room, ICON = :icon, IP = :ip, NAME = :name, 
					CHANNEL = :channel WHERE ID = :id;");
			}
			
			//DIY Sensor bearbeiten bzw. hinzufügen
			break;
		case "DIY Schalter":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "ip" => $dataArray['ip'], "channel" => $dataArray['channel'],
				"icon" => $dataArray['icon']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'DIY_SENSORS' (RAUM, ICON, IP, NAME, CHANNEL)
									VALUES (:room, :icon, :ip, :name, :channel);");
			}	
			else{
				$query = $db->prepare("UPDATE 'DIY_SENSORS' SET RAUM = :room, ICON = :icon, IP = :ip, NAME = :name, CHANNEL = :channel
									WHERE ID = :id;");
			}
			
			//DIY Schalter bearbeiten bzw. hinzufügen
			break;
		case "DIY Gerät":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "ip" => $dataArray['ip'],
				"icon" => $dataArray['icon']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'DIY_DEVICES' (NAME, ROOM, IP, ICON) VALUES (:name, :room, :ip, :icon);");
			}	
			else{
				$query = $db->prepare("UPDATE 'DIY_DEVICES' SET ROOM = :room, ICON = :icon, IP = :ip, NAME = :name WHERE DEVICE = :id;");
			}
			
			//DIY Gerät bearbeiten bzw. hinzufügen
			break;
		//MAX! Thermostat
		case "MAX! Thermostat":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "icon" => $dataArray['icon']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'MAX_THERMOSTATS' (NAME, RAUM, ID, ICON) VALUES (:name, :room, :id, :icon);");
			}	
			else{
				$query = $db->prepare("UPDATE 'MAX_THERMOSTATS' SET RAUM = :room, ICON = :icon, NAME = :name WHERE ID = :id;");
			}
			
			//MAX! Thermostat bearbeiten bzw. hinzufügen
			break;
		//IP Kamera
		case "IP Kamera":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "icon" => $dataArray['icon']);
			
			if($addNew){
				$query = $db->prepare("INSERT INTO 'IP_CAMERA' (NAME, RAUM, ID, ICON) VALUES (:name, :room, :id, :icon);");
			}	
			else{
				$query = $db->prepare("UPDATE 'IP_CAMERAS'
				SET TYPE = :type, IP = :ip, PORT = :port , PATH = :path, USERNAME = :username, PASSWORD = :password,
				RECORD_FOOTAGE = :recordfootage , AUTO_DELETE = :autodelete, MOTION_DETECTION_THRESHOLD = :detectionthreshold,
				LOCATION = :room, ICON = :icon, NAME = :name WHERE ID = :id;");
			}
			
			//IP Kamera bearbeiten bzw. hinzufügen
			break;
		//Wake on Lan Gerät
		case "Wake on Lan Gerät":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "icon" => $dataArray['icon'], "mac" => $dataArray['mac']);
			
			//Wake on Lan Gerät bearbeiten bzw. hinzufügen
			if($addNew){
				$query = $db->prepare("INSERT INTO 'WAKE_ON_LAN' (DEVICE, NAME, LOCATION, ICON, MAC_ADDRESS) VALUES (null, :name, :room, :icon, :mac);");
			}
			else{
				$query = $db->prepare("UPDATE 'WAKE_ON_LAN' SET LOCATION = :room, ICON = :icon, NAME = :name, MAC_ADDRESS = :mac WHERE DEVICE = :id;");
			}
			
			break;
		//XBOX One
		case "XBOX One":
			$paramArray = array("room" => $dataArray['room'], "name" => $dataArray['name'], "icon" => $dataArray['icon'], "ip" => $dataArray['ip'],
			"liveid" => $dataArray['liveid']);
			
			//Wake on Lan Gerät bearbeiten bzw. hinzufügen
			if($addNew){
				$query = $db->prepare("INSERT INTO 'XBOX_ONE_WOL' (ID, NAME, LOCATION, ICON, IP_ADDRESS, XBOX_LIVE_ID) VALUES (null, :name, :room, :icon, :ip, :liveid);");
			}
			else{
				$query = $db->prepare("UPDATE 'XBOX_ONE_WOL' SET LOCATION = :room, ICON = :icon, NAME = :name, IP_ADDRESS = :ip, XBOX_LIVE_ID = :liveid WHERE DEVICE = :id;");
			}
			
			break;
		default:
			return "nosuchtype";
	}
}

function getDeviceData($type, $id, $db){
	if(!hasPermission("admin", $db)){
		return "noadmin";
	}
	
	$type = utf8_decode($type);
	
	//Räume abfragen
	$roomArray = array();
	
	foreach(getRooms($db)['rooms'] as $room){
		$roomArray[$room['location']] = $room['name'];
	}
	
	$queryParams = array("id" => $id);
	
	$devicedata = array();
	
	switch($type){
		//433 Mhz
		case "Funksteckdose":
			$dbQuery = $db->prepare("SELECT * FROM 'funksteckdosen' WHERE DEVICE == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['ROOM']], "location" => $query['ROOM'], "name" => $query['NAME'], "hauscode" => $query['HAUSCODE'],
				"icon" => $query['ICON'], "steckdosennummer" => $query['STECKDOSENNUMMER']);
			}
			break;
			
		//DIY
		case "DIY Gerät":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_DEVICES' WHERE DEVICE == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['ROOM']], "location" => $query['ROOM'], "name" => $query['NAME'],
				"ip" => $query['IP'],"icon" => $query['ICON']);
			}
			break;
		case "DIY Sensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
				"ip" => $query['IP'], "save_data" => $query['SAVE_DATA'], "einheit" => $query['EINHEIT'], "channel" => $query['CHANNEL']);
			}
		case "DIY Fensterkontakt":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_REEDSENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
				"ip" => $query['IP'], "channel" => $query['CHANNEL']);
			}
			break;
		case "DIY Anwesenheitssensor":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_PRESENCESENSORS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
				"ip" => $query['IP'], "channel" => $query['CHANNEL']);
			}
			break;
		case "DIY Schalter":
			$dbQuery = $db->prepare("SELECT * FROM 'DIY_SWITCHES' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'],
				"icon" => $query['ICON'], "ip" => $query['IP'], "channel" => $query['CHANNEL']);
			}
			break;
			
		//Z-Wave
		case "Z-Wave Sensor":
			$dbQuery = $db->prepare("SELECT * FROM 'ZWAVE_SENSOREN' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['SHORTFORM'],
				"icon" => $query['ICON'], "save_data" => $query['SAVE_DATA'], "einheit" => $query['EINHEIT']);
			}
			break;
		case "Z-Wave Thermostat":
			$dbQuery = $db->prepare("SELECT * FROM 'ZWAVE_THERMOSTATS' WHERE THERMOSTAT_ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON']);
			}
			break;
		case "Z-Wave Strommesser":
			$dbQuery = $db->prepare("SELECT * FROM 'ZWAVE_POWER_METER' WHERE DEVICE_ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['ROOM_ID']], "location" => $query['ROOM_ID'], "name" => $query['DEVICE_NAME'], "icon" => $query['ICON']);
			}
			break;
		case "Z-Wave Schalter":
			$dbQuery = $db->prepare("SELECT * FROM 'ZWAVE_SWITCHES' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['LOCATION']], "location" => $query['LOCATION'], "name" => $query['NAME'], "icon" => $query['ICON']);
			}
			break;
		case "Z-Wave Dimmer":
			$dbQuery = $db->prepare("SELECT * FROM 'ZWAVE_DIMMER' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['LOCATION']], "location" => $query['LOCATION'], "name" => $query['NAME'], "icon" => $query['ICON']);
			}
			break;
		//MAX! Thermostat
		case "MAX! Thermostat":
			$dbQuery = $db->prepare("SELECT * FROM 'MAX_THERMOSTATS' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['RAUM']], "location" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON']);
			}
			break;
		//IP Kamera
		case "IP Kamera":
			$dbQuery = $db->prepare("SELECT * FROM 'IP_CAMERA' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['LOCATION']], "location" => $query['RAUM'], "name" => $query['NAME'],
				"icon" => $query['ICON'], "ip" => $query['IP'], "port" => $query['PORT'], "path" => $query['PATH'],
				"username" => $query['USERNAME'], "password" => $query['PASSWORD'], "recordfootage" => $query['RECORD_FOOTAGE'],
				"autodelete" => $query['AUTO_DELETE'], "detectionthreshold" => $query['MOTION_DETECTION_THRESHOLD']);
			}
			break;
		//Wake on Lan Gerät
		case "Wake on Lan Gerät":
			$dbQuery = $db->prepare("SELECT * FROM 'WAKE_ON_LAN' WHERE DEVICE == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['LOCATION']], "location" => $query['LOCATION'], "name" => $query['NAME'],
				"icon" => $query['ICON'], "mac" => $query['MAC_ADDRESS']);
			}
			break;
		//XBOX One
		case "XBOX One":
			$dbQuery = $db->prepare("SELECT * FROM 'XBOX_ONE_WOL' WHERE ID == :id");
			$dbQuery->execute($queryParams);
			
			foreach($dbQuery->fetchAll(PDO::FETCH_ASSOC) as $query){
				$devicedata = array("room" => $roomArray[$query['LOCATION']], "location" => $query['LOCATION'], "name" => $query['NAME'],
				"icon" => $query['ICON'], "ip" => $query['IP_ADDRESS'], "liveid" => $query['XBOX_LIVE_ID']);
			}
			break;
		default:
			return "nosuchtype";
	}
	
	return array("devicedata" => $devicedata);
}

function deleteDevice($type, $devicekey, $db){
	if(!hasPermission("admin", $db)){
		return "noadmin";
	}
	
	$type = utf8_decode($type);
	
	switch($type){
		//433 Mhz
		case "Funksteckdose":
			$result = $db->prepare("DELETE FROM funksteckdosen WHERE DEVICE == :device");
			break;
		//DIY
		case "DIY Gerät":
			$result = $db->prepare("DELETE FROM DIY_DEVICES WHERE DEVICE == :device");
			break;
		case "DIY Fensterkontakt":
			$result = $db->prepare("DELETE FROM DIY_REEDSENSORS WHERE ID == :device");
			break;
		case "DIY Sensor":
			$result = $db->prepare("DELETE FROM DIY_SENSORS WHERE ID == :device");
			break;
		case "DIY Schalter":
			$result = $db->prepare("DELETE FROM DIY_SWITCHES WHERE ID == :device");
			break;
		//Z-Wave
		case "Z-Wave Sensor":
			$result = $db->prepare("DELETE FROM ZWAVE_SENSOREN WHERE ID == :device");
			break;
		case "Z-Wave Thermostat":
			$result = $db->prepare("DELETE FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID == :device");
			break;
		case "Z-Wave Stromzähler":
			$result = $db->prepare("DELETE FROM ZWAVE_POWER_METER WHERE DEVICE_ID == :device");
			break;
		case "Z-Wave Dimmer":
			$result = $db->prepare("DELETE FROM ZWAVE_DIMMER WHERE ID == :device");
			break;
		case "Z-Wave Schalter":
			$result = $db->prepare("DELETE FROM ZWAVE_SWITCHES WHERE ID == :device");
			break;
		case "Wake on Lan Gerät":
			$result = $db->prepare("DELETE FROM WAKE_ON_LAN WHERE DEVICE == :device");
			break;
		case "XBOX One":
			$result = $db->prepare("DELETE FROM XBOX_ONE_WOL WHERE ID == :device");
			break;
		default:
		return "nosuchtype";
	}
	
	$result->execute(array("device" => $devicekey));
	
	if($result == true) return "ok";
	else return "error";
}
'''

def get_device_data(username, type, id, db):
    if not has_permission(username, "admin", db):
        return {'result': 'noadmin'}

    room_array = {}

    for room in get_rooms(username, db)['rooms']:
        room_array[room['location']] = room['name']

    Logger.log(room_array)

    with db:
        cur = db.cursor()

        param_array = {'id': id}

        device_data = {}

        #Funksteckdose
        if type == FUNKSTECKDOSE:
            cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                           'hauscode': data['HAUSCODE'], 'icon': data['ICON'], 'steckdosennummer': data['STECKDOSENNUMMER']}
        #Z-Wave
        elif type == ZWAVE_POWER_METER:
            cur.execute("SELECT * FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['ROOM_ID'])], 'location': data['ROOM_ID'], 'name': data['DEVICE_NAME'],
                           'is_reset_daily': (data['IS_RESET_DAILY']==1), 'icon': data['ICON']}
        elif type == ZWAVE_SENSOR:
            cur.execute("SELECT * FROM ZWAVE_SENSOREN WHERE ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['SHORTFORM'],
                           'save_data': data['SAVE_DATA'], 'icon': data['ICON'],
                           'sensor_type': data['SENSOR_TYPE'], 'sensor_type_name': SENSOR_TYPE_MAP[data['SENSOR_TYPE']]['name']}
        elif type == ZWAVE_THERMOSTAT:
            cur.execute("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == ZWAVE_SWITCH:
            cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == ZWAVE_DIMMER:
            cur.execute("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        #MQTT-Sensor
        elif type == MQTT_SENSOR:
            cur.execute("SELECT * FROM MQTT_SENSORS, MQTT_DEVICES WHERE MQTT_SENSORS.DEVICE_ID = MQTT_DEVICES.ID AND MQTT_SENSORS.ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'], 'icon': data['ICON'],
                           'topic': data['TOPIC'], 'key': data['KEY'], 'sensor_type': data['TYPE'],
                           'sensor_type_name': SENSOR_TYPE_MAP[data['TYPE']]['name'], 'save_data': data['SAVE_DATA']==1}
        #IP Kamera
        elif type == IP_CAM:
            cur.execute("SELECT * FROM ZWAVE_DIMMER WHERE ID = :id", param_array)

            data = cur.fetchone()

            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'ip': data['IP'], 'port': data['PORT'], 'path': data['PATH'],
                           'username': data['USERNAME'], 'password': data['PASSWORD'], 'recordfootage': data['RECORD_FOOTAGE'],
                           'autodelete': data['AUTO_DELETE'], 'detectionthreshold': data['MOTION_DETECTION_THRESHOLD']}
        #URL-Toggle
        elif type == URL_TOGGLE:
            cur.execute("SELECT * FROM URL_TOGGLE WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'toggle_url': data['TOGGLE_URL']}
        #URL-Switch
        elif type == URL_SWITCH:
            cur.execute("SELECT * FROM URL_SWITCH_BINARY WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'on_url': data['URL_ON'], 'off_url': data['URL_OFF'],
                           'get_state_url': data['URL_GET_STATE']}
        #URL RGB-Licht
        elif type == URL_RGB_LIGHT:
            cur.execute("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'url': data['URL']}
        #Philips Hue RGB-Licht
        elif type == PHILIPS_HUE_LIGHT:
            cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'type': data['TYPE']}
        #MAX Thermostat
        elif type == MAX_THERMOSTAT:
            cur.execute("SELECT * FROM MAX_THERMOSTATS WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['RAUM'])], 'location': data['RAUM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        #Wake on Lan
        elif type == WAKE_ON_LAN:
            cur.execute("SELECT * FROM WAKE_ON_LAN WHERE DEVICE = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'mac': data['MAC_ADDRESS']}
        #XBOX Wake on Lan
        elif type == XBOX_ONE_WOL:
            cur.execute("SELECT * FROM XBOX_ONE_WOL WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON'], 'ip': data['IP_ADDRESS'], 'liveid': data['XBOX_LIVE_ID']}
        #Rademacher
        elif type == RADEMACHER_THERMOSTAT:
            cur.execute("SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['ROOM'])], 'location': data['ROOM'], 'name': data['NAME'],
                           'icon': data['ICON']}
        elif type == RADEMACHER_BLIND_CONTROL:
            cur.execute("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", param_array)

            data = cur.fetchone()
            device_data = {'room': room_array[int(data['LOCATION'])], 'location': data['LOCATION'], 'name': data['NAME'],
                           'icon': data['ICON']}
        else:
            return {'result': 'nosuchtype'}

        return {'devicedata': device_data}


from homevee.exceptions import AbstractFunctionCallException
from homevee.items.Device import Device


class Switch(Device):
    def __init__(self, name, icon, location, id=None, mode=False):
        super(Switch, self).__init__(name, icon, location, id=id)

        if mode == 1:
            self.mode = True
        else:
            self.mode = False

    def set_mode(self, mode, db=None):
        #TODO sicherstellen, dass mode immer boolean ist (Ticket #26)
        if mode == "1" or mode == 1 or mode == True:
            mode = True
        else:
            mode = False

        if(self.update_mode(mode, db)):
            self.mode = mode
            self.save_to_db(db)

    def update_mode(self, mode, db=None):
        raise AbstractFunctionCallException("Switch.update_mode() is abstract")

    #@staticmethod
    #def get_all(location=None, db=None):
    #    # get all devices of all types
    #    devices = []
    #    devices.extend(Switch.get_all(location, db))
    #    return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.Modules.ShoppingListModule import VoiceShoppingListModule
from homevee.VoiceAssistant.voice_patterns import *


class VoiceAddToShoppingListModule(VoiceShoppingListModule):
    def get_pattern(self, db):
        return PATTERN_ADD_SHOPPING_LIST

    def get_label(self):
        return "getshoppinglist"

    def run_command(self, username, text, context, db):
        return self.add_to_shopping_list(username, text, context, db)

    def add_to_shopping_list(self, username, text, context, db):
        '''words = text.split(" ")

        for i in range(0, len(words)):
            word = words[i]



        if item_count > 1:
            answer_data = [
                ['Ok']
            ]
        else:
            answer_data = [
                ['Ok']
            ]

        output = generate_string(answer_data)'''

        output = "Add Shopping List"

        return {'msg_speech': output, 'msg_text': output}




#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *

def get_all_scenes(username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM ROOMS")

        rooms = []

        for room in cur.fetchall():
            if not has_permission(username, room['LOCATION'], db):
                continue

            scenes = get_scenes(username, room['LOCATION'], db)

            scenes = scenes['scenes']

            if(len(scenes) is not 0):
                room_item = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'], 'scenes': scenes}
                rooms.append(room_item)

        return {'rooms': rooms}

def get_scenes(username, location, db):
    scenes = []

    if has_permission(username, location, db):
        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM SCENES WHERE ROOM = :location",{'location': location})

            for item in cur.fetchall():
                scenes.append({'id': item['ID'], 'name': item['NAME'], 'action_data': item['ACTION_DATA'],
                              'location': item['ROOM']})

    return {'scenes': scenes}

def add_edit_scene(username, id, name, location, action_data, db):
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        if(add_new):
            cur.execute("INSERT INTO SCENES (NAME, ROOM, ACTION_DATA) VALUES (:name, :room, :actions)",
                        {'name': name, 'room': location, 'actions': action_data})

            return Status(type=STATUS_OK).get_dict()

        else:
            cur.execute("UPDATE SCENES SET NAME = :name, ROOM = :location, ACTION_DATA = :actions WHERE ID = :id",
                {'name': name, 'location': location, 'actions': action_data, 'id': id})

            return Status(type=STATUS_OK).get_dict()

def delete_scene(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM SCENES WHERE ID = :id", {'id': id})

        return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *


def get_all_api_key_data(username, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    services = [
        {'servicename': 'Open Weather Map',
         'description': 'Open Weather Map stellt Wetterdaten und -vorhersagen bereit.', 'registerkeyurl': '',
         'servicelogourl': 'https://openweathermap.org/themes/openweathermap/assets/vendor/owm/img/logo_OpenWeatherMap_orange.svg'},
        {'servicename': 'IMDB', 'description': 'IMDB ist eine internationale Filmdatenbank', 'registerkeyurl': '',
         'servicelogourl': 'http://ia.media-imdb.com/images/G/01/imdb/images/mobile/imdb-logo-responsive@2-868559777._CB514893749_.png'}
    ]

    output = []

    for service in services:
        service['key'] = get_api_key(service['servicename'], db)

        output.append(service)

    return {'apikeys': output}

def get_api_key(servicename, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT API_KEY FROM API_KEYS WHERE SERVICE_NAME = :name", {'name': servicename})

        try:
            service_data = cur.fetchone()
            key = service_data['API_KEY']
        except TypeError as e:
            #key doesn't exist
            return None

        return key

def set_api_key(username, service, api_key, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()

        params = {'key': api_key, 'name': service}

        cur.execute("UPDATE OR IGNORE API_KEYS SET API_KEY = :key WHERE SERVICE_NAME = :name;", params)
        cur.execute("INSERT OR IGNORE INTO API_KEYS (API_KEY, SERVICE_NAME) VALUES (:key, :name);", params)

        return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.Functions.Automations import Automation
from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_database_con


class TimedAutomationsCronjob(IntervalCronjob, Automation):
    def __init__(self):
        super(TimedAutomationsCronjob, self).__init__(task_name="TimedAutomationsCronjob", interval_seconds=60)

    def task_to_do(self, *args):
        self.run_timed_automations()

    def run_timed_automations(self):
        # print "running automations"
        db = get_database_con()
        with db:
            cur = db.cursor()

            today = datetime.datetime.today()

            hour = today.hour
            minute = today.minute

            hour_string = str(hour)
            if (hour < 10):
                hour_string = "0" + hour_string

            minute_string = str(minute)
            if (minute < 10):
                minute_string = "0" + minute_string

            time_string = hour_string + ":" + minute_string

            # print time_string

            cur.execute(
                'SELECT * FROM AUTOMATION_DATA, AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_DATA.ID = AUTOMATION_RULE_ID AND TYPE = "time" AND IS_ACTIVE = "true" AND VALUE = :val',
                {'val': time_string})

            automations = cur.fetchall()

            self.run_automations(automations, db)

        return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback

from homevee.DeviceAPI import rademacher_homepilot
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *
from homevee.utils.device_types import *


def set_blinds(username, type, id, new_position, db, check_user=True):
    try:
        with db:
            cur = db.cursor()

            if type == RADEMACHER_BLIND_CONTROL:
                cur.execute("SELECT LOCATION FROM HOMEPILOT_BLIND_CONTROL WHERE ID = :id", {'id': id})
                data = cur.fetchone()
                if check_user and not has_permission(username, data['LOCATION'], db):
                    return Status(type=STATUS_NO_PERMISSION).get_dict()

                return rademacher_homepilot.blinds_control.control_blinds(id, new_position, db)
            else:
                return Status(type=STATUS_NO_SUCH_TYPE).get_dict()
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
        return Status(type=STATUS_ERROR).get_dict()

def set_room_blinds(username, room, new_position, db):
    with db:
        cur = db.cursor()
        devices = get_blinds(username, room, db)
        
        for device in devices['blinds']:
            id = device['id']
            type = device['type']
            result = set_blinds(username, type, id, new_position, db)
            if 'result' not in result or result['result'] != 'ok':
                return Status(type=STATUS_ERROR).get_dict()
        return Status(type=STATUS_OK).get_dict()
        
def get_all_blinds(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM ROOMS")
        rooms = cur.fetchall()
        blinds = []
        for room in rooms:
            if not has_permission(username, room['LOCATION'], db):
                continue
            room_blinds = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'],
                                'blind_array': get_blinds(username, room['LOCATION'], db)}
            blinds.append(room_blinds)
        return {'blinds': blinds}

def get_blinds(username, location, db):
    if not has_permission(username, location, db):
        return {'result': 'nopermission'}

    with db:
        blinds = []

        cur = db.cursor()

        #Rademacher HomePilot
        cur.execute("SELECT * FROM HOMEPILOT_BLIND_CONTROL WHERE LOCATION = :location",
                    {'location': location})
        for item in cur.fetchall():
            value = int(item['LAST_POS'])

            if value is None:
                value = 0

            blinds.append({'name': item['NAME'], 'id': item['ID'], 'location': location,
                           'icon': item['ICON'], 'value': value, 'type': RADEMACHER_BLIND_CONTROL})

        #Andere Gerätetypen

        return {'blinds': blinds}



#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import io
import json
import os
import time
import traceback
from _thread import start_new_thread

from PIL import Image

from homevee.DeviceAPI.get_modes import get_modes
from homevee.DeviceAPI.heating import get_thermostats
from homevee.DeviceAPI.rgb_control import get_rgb_device
from homevee.DeviceAPI.set_modes import set_modes
from homevee.Functions.condition_actions.actions import run_scene
from homevee.Functions.tensorflow_functions import ar_control_predict, ar_control_retrain
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *
from homevee.utils import constants
from homevee.utils.constants import DATA_DIR
from homevee.utils.database import get_server_data, get_database_con, set_server_data
from homevee.utils.device_types import *
from homevee.utils.file_utils import create_image

IS_TRAINING_TAG = "AR_CONTROL_TRAINING_RUNNING"

def upload_images(username, data, img_class, db):
    images = json.loads(data)

    counter = 0

    if (img_class == "-1" or img_class == -1):
        img_class = None

    for image in images:
        filename = "device-" + str(int(time.time())) + "_" + str(counter)

        counter += 1

        image_path = create_image(filename, "ar_control", image, optimize=True)

        with db:
            cur = db.cursor()

            cur.execute("INSERT INTO AR_CONTROL_LEARNING_DATA (PATH, CONTROL_KEY) VALUES (:path, :class)",
                        {'path': image_path, 'class': img_class})
        #print image

    return Status(type=STATUS_OK).get_dict()

def save_ar_control_class(username, id, data, classname, db):
    if not has_permission(username, 'admin', db):
        return {'result': 'noadmin'}

    with db:
        cur = db.cursor()

        params = {'name': classname, 'data': data}

        if id is not None and id is not "" and id != "-1":
            params['id'] = id
            cur.execute("UPDATE AR_CONTROL_CLASSES SET NAME = :name, CONTROL_DATA = :data WHERE ID = :id",
                        params)
        else:
            cur.execute("INSERT INTO AR_CONTROL_CLASSES (NAME, CONTROL_DATA) VALUES (:name, :data)",
                        params)

    return Status(type=STATUS_OK).get_dict()

def ar_control(username, image_data, db):
    filename = "device-"+str(int(time.time()))

    image_path = create_image(filename, "ar_control", image_data, optimize=True)

    with db:
        cur = db.cursor()

        cur.execute("INSERT INTO AR_CONTROL_LEARNING_DATA (PATH) VALUES (:path)",
                    {'path': image_path})

    if not has_ar_control_classes(db):
        return "Noch nicht trainiert."


    predictions = ar_control_predict.predict(os.path.join(constants.DATA_DIR, image_path))

    class_data = get_ar_class_data(predictions[0]['prediction'], db)

    Logger.log(image_path)
    Logger.log(class_data['NAME'] + ": " + str(predictions[0]['confidence']*100))
    #Logger.log(class_data)

    data = run_ar_command(username, class_data['CONTROL_DATA'], db)

    return data

def has_ar_control_classes(db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT COUNT(*) FROM AR_CONTROL_CLASSES")
        result = cur.fetchone()
        count = result['COUNT(*)']

        return count > 0

def run_ar_command(username, data, db):
    data = json.loads(data)

    if data['type'] == "scene":
        run_scene(username, data['id'], db)
    elif data['type'] in [FUNKSTECKDOSE, URL_TOGGLE, URL_SWITCH, ZWAVE_SWITCH]:
        mode = get_modes(username, None, data['type'], data['id'], db)

        if mode is 0:
            mode = 1
        else:
            mode = 0

        set_modes(username, data['type'], data['id'], str(mode), db)

        return {'action': None}
    elif data['type'] in [MAX_THERMOSTAT, ZWAVE_THERMOSTAT]:
        thermostat_data = get_thermostats(username, None, data['type'], data['id'], db)
        return {'action': 'heating', 'data': thermostat_data, 'type': data['type'], 'id': data['id']}
    elif data['type'] in [PHILIPS_HUE_LIGHT, URL_RGB_LIGHT]:
        rgb_data = get_rgb_device(username, data['type'], data['id'], db)
        return {'action': 'rgb', 'data': rgb_data, 'type': data['type'], 'id': data['id']}
    else:
        return {'error': 'nosuchaction'}

def get_ar_class_data(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM AR_CONTROL_CLASSES WHERE ID = :id;", {'id': id})

        return cur.fetchone()

def start_ar_training(username, db):
    if not has_permission(username, 'admin', db):
        return {'result': 'noadmin'}

    is_training = get_server_data(IS_TRAINING_TAG, db)

    if (is_training is not None and is_training == "true"):
        return {'result': 'alreadytraining'}

    start_new_thread(training_thread, (username, None))

    return Status(type=STATUS_OK).get_dict()


def training_thread(username, d):
    db = get_database_con()

    set_server_data(IS_TRAINING_TAG, "true", db)

    try:
        ar_control_retrain.ar_training(username)
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()

    set_server_data(IS_TRAINING_TAG, "false", db)

def get_ar_control_classes(db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM 'AR_CONTROL_CLASSES'")

        classes = []

        for row in cur.fetchall():
            item = {'id': int(row['ID']), 'name': row['NAME'], 'data': row['CONTROL_DATA']}
            classes.append(item)

        item = {'id': None, 'name': 'Nicht zugeordnet'}
        classes.append(item)

        return {'classes': classes}

def get_ar_control_class_images(image_class, show, offset, db):
    param_array = {'limit': show, 'offset': offset}

    with db:
        cur = db.cursor()

        if image_class is None or image_class == "-1" or image_class == -1:
            cur.execute("SELECT * FROM 'AR_CONTROL_LEARNING_DATA' WHERE CONTROL_KEY IS NULL OR CONTROL_KEY = -1 LIMIT :limit OFFSET :offset",
                        param_array)
        else:
            param_array['key'] = image_class
            cur.execute("SELECT * FROM 'AR_CONTROL_LEARNING_DATA' WHERE CONTROL_KEY = :key LIMIT :limit OFFSET :offset",
                        param_array)

        images = []

        for img in cur.fetchall():
            images.append(img['ID'])

        return {'images': images}

def change_ar_image_class(ids, newclass, db):
    if newclass == "-1":
        newclass = None

    ids = json.loads(ids)

    Logger.log(ids)

    with db:
        cur = db.cursor()

        for id in ids:
            cur.execute("UPDATE 'AR_CONTROL_LEARNING_DATA' SET CONTROL_KEY = :key WHERE ID == :id;",
                        {'key': newclass, 'id': id})

    return Status(type=STATUS_OK).get_dict()

def delete_ar_control_images(ids, db):
    #todo

    ids = json.loads(ids)

    with db:
        cur = db.cursor()

        for id in ids:
            Logger.log("deleting: "+str(id))
            cur.execute("SELECT * FROM AR_CONTROL_LEARNING_DATA WHERE ID == :id;",
                        {'id': id})
            item = cur.fetchone()
            if(item is not None):
                rel_path = item['PATH']

                image_path = os.path.join(DATA_DIR, rel_path)
                if os.path.exists(image_path):
                    os.remove(image_path)
                else:
                    Logger.log(image_path+" not found")

    with db:
        cur = db.cursor()
        for id in ids:
            cur.execute("DELETE FROM AR_CONTROL_LEARNING_DATA WHERE ID == :id;",
                        {'id': id})

    return Status(type=STATUS_OK).get_dict()

def get_ar_control_image(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT PATH FROM 'AR_CONTROL_LEARNING_DATA' WHERE ID == :id",
                    {'id': id})

        rel_path = cur.fetchone()['PATH']

        path = os.path.join(DATA_DIR, rel_path)
        Logger.log(("Path: "+path))

        im = Image.open(path)

        size = 50

        image_dimensions = size, size

        im.thumbnail(image_dimensions, Image.ANTIALIAS)

        buffer = io.BytesIO()
        im.save(buffer, format="JPEG")
        encoded_string = base64.b64encode(buffer.getvalue())
        encoded_string = encoded_string.decode('utf-8')
        im.close()

        #with open(path, "rb") as image_file:
        #    encoded_string = base64.b64decode(image_file.read())

    return {'imagedata': encoded_string}

def get_performance_settings(username, db):
    if not has_permission(username, 'admin', db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    prefix = "AR_"

    tf_data = {}

    with db:
        cur = db.cursor()

        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

        for index in indices:
            cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                        {'key': prefix+index})

            result = cur.fetchone()
            if(result is not None and 'VALUE' in result):
                tf_data[index] = result['VALUE']

    return {'data': tf_data, 'graph_sizes': ar_control_retrain.get_graph_sizes(),
            'image_sizes' : ar_control_retrain.get_image_sizes()}

def set_performance_settings(username, data, db):
    if not has_permission(username, 'admin', db):
        return {'result': 'noadmin'}

    prefix = "AR_"

    data = json.loads(data)

    with db:
        cur = db.cursor()

        indices = ['TF_TRAINING_STEPS', 'TF_MODEL_SIZE', 'TF_MODEL_IMAGE_SIZE']

        for index in indices:
            param_array = {'value': data[index], 'key': prefix+index}

            cur.execute("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key;", param_array)

            cur.execute("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES (:value, :key);", param_array)

    return {'result':'ok'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import time

from homevee.Helper.helper_functions import has_permission, get_rooms
from homevee.utils.device_types import *
from homevee.items.Status import *


def get_power_usage_devices(username, room, db):
    devices = []

    for item in get_energy_device_data(username, room, db):
        try:
            if item['value'] is not None:
                item['value'] = str(round(item['value'], 2)) + " kWh"
            else:
                item['value'] = "N/A"
        except:
            item['value'] = "N/A"
        devices.append(item)

    return devices

def get_energy_device_data(username, room, db):
    devices = []

    if not has_permission(username, room, db):
        return {'result': 'nopermission'}

    with db:
        cur = db.cursor()

        # Z-Wave Stromzähler
        cur.execute("SELECT * FROM ZWAVE_POWER_METER WHERE ROOM_ID = :room", {'room': room})
        for item in cur.fetchall():
            try:
                value = float(item['VALUE'])
            except:
                value = "N/A"

            devices.append({'name': item['DEVICE_NAME'], 'id': item['DEVICE_ID'], 'devicetype': ZWAVE_POWER_METER,
                            'icon': item['ICON'], 'value': value, 'location': room})

    return devices

def get_current_power_usage(username, db):
    devices = []

    for room in get_rooms(username, db)['rooms']:
        devices.extend(get_energy_device_data(username, room['location'], db))

    return devices

def get_current_room_power_usage(username, db):
    rooms = {}

    for room in get_rooms(username, db)['rooms']:
        usage = 0
        for device in get_energy_device_data(username, room['location'], db):
            if device['value'] is not None:
                usage += device['value']

        rooms[str(room['location'])] = {'usage': usage,
                                        'devices': get_energy_device_data(username, room['location'], db)}

    return rooms

def get_current_device_power_usage(username, room, type, id, db):
    devices = get_power_usage_devices(username, room, db)

    for device in devices:
        if device['devicetype'] == type and device['id'] == id:
            return device['VALUE']
    return None

def get_energy_data(username, room, devicetype, deviceid, von, bis, db):
    von = datetime.datetime.strptime(von, '%d.%m.%Y')
    bis = datetime.datetime.strptime(bis, '%d.%m.%Y')

    if((room is None or room == "")
            and (devicetype is None or devicetype == "")
            and (deviceid is None or deviceid == "")):
        with db:
            room_data = get_current_room_power_usage(username, db)

            rooms_sum = 0
            for room_item in room_data:
                rooms_sum += room_data[room_item]['usage']

            cur = db.cursor()

            data = {}

            cur.execute("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM ENERGY_DATA WHERE DATE >= :von AND DATE <= :bis",
                        {'von': von, 'bis': bis})
            result = cur.fetchone()
            data['totalusage'] = result['POWER_USAGE']+rooms_sum
            data['cost'] = get_power_cost(username, db)

            percentage = []
            cur.execute("SELECT ROOMS.NAME, SUM(POWER_USAGE) AS USAGE, ROOM_ID FROM ROOM_ENERGY_DATA, ROOMS WHERE ROOM_ID = ROOMS.LOCATION AND DATE >= :von AND DATE <= :bis GROUP BY ROOM_ID",
                        {'von': von, 'bis': bis})

            for item in cur.fetchall():
                energy_item = {'type': '', 'id': '', 'location': item['ROOM_ID'],
                                    'name': item['NAME'], 'usage': float(item['USAGE'])+room_data[str(item['ROOM_ID'])]['usage']}
                percentage.append(energy_item)

            data['usagepercentage'] = percentage

            data['usagecourse'] = get_energy_course(username, room, von, bis, db)
            data['usagecourse'].append({'date': time.strftime('%d.%m.%Y'), 'usage': rooms_sum})

            return {'energydata': data}
    elif(type is not None and type != '' and deviceid is not None and deviceid != '' and room is not None and room != ''):
        return get_device_energy_data(username, room, devicetype, deviceid, von, bis, db)
    else:
        return get_room_energy_data(username, room, von, bis, db)

def get_room_energy_data(username, room, von, bis, db):
    data = {}

    with db:
        cur = db.cursor()

        room_data = get_current_room_power_usage(username, db)[room]['devices']
        room_data_dict = {}
        room_total_usage = 0
        for item in room_data:
            room_total_usage += item['value']
            room_data_dict[str(item['id'])+str(item['devicetype'])] = item['value']

        cur.execute("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM ROOM_ENERGY_DATA WHERE ROOM_ID = :room AND DATE >= :von AND DATE <= :bis",
                    {'room': room, 'von': von, 'bis': bis})
        result = cur.fetchone()

        power_usage = 0

        if(result['POWER_USAGE'] is not None):
            power_usage = result['POWER_USAGE']

        data['totalusage'] = float(power_usage)+room_total_usage
        data['cost'] = get_power_cost(username, db)

        percentage = []

        device_types = []

        device_types.append("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID FROM DEVICE_ENERGY_DATA, ZWAVE_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = \""+ZWAVE_POWER_METER+"\" AND LOCATION = :room AND DATE >= :von AND DATE <= :bis GROUP BY DEVICE_NAME")

        #device_types.append("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID	FROM DEVICE_ENERGY_DATA, DIY_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = "+DIY+" AND LOCATION = :room AND DATE >= :von AND DATE <= :bis GROUP BY DEVICE_NAME")

        print("SELECT SUM(POWER_USAGE) AS POWER_USAGE, DEVICE_NAME, DEVICE_TYPE, DEVICE_ENERGY_DATA.DEVICE_ID FROM DEVICE_ENERGY_DATA, ZWAVE_POWER_METER WHERE LOCATION = ROOM_ID AND DEVICE_TYPE = \""+ZWAVE_POWER_METER+"\" AND LOCATION = \""+str(room)+"\" AND DATE >= \""+str(von.strftime('%Y-%m-%d'))+"\" AND DATE <= \""+str(bis.strftime('%Y-%m-%d'))+"\" GROUP BY DEVICE_NAME")

        for device_type in device_types:
            cur.execute(device_type, {'room': room, 'von': von, 'bis': bis})

            for item in cur.fetchall():
                print(item)

                current_item_value = room_data_dict[str(item['DEVICE_ID'])+str(item['DEVICE_TYPE'])]

                percentage.append({'type': item['DEVICE_TYPE'],'id': item['DEVICE_ID'],
                                   'location': room, 'name': item['DEVICE_NAME'], 'usage': float(item['POWER_USAGE'])+current_item_value})

        data['usagepercentage'] = percentage

        data['usagecourse'] = get_energy_course(username, room, von, bis, db)
        data['usagecourse'].append({'date': time.strftime('%d.%m.%Y'), 'usage': room_total_usage})

        return {'energydata': data}

def get_device_energy_data(username, room , devicetype, deviceid, von, bis, db):
    data = {}

    with db:
        cur = db.cursor()

        cur.execute("SELECT SUM(POWER_USAGE) AS POWER_USAGE FROM DEVICE_ENERGY_DATA WHERE DEVICE_TYPE = :type AND DEVICE_ID = :id AND DATE >= :von AND DATE <= :bis",
                    {'type': devicetype, 'id': deviceid, 'von':von, 'bis': bis})

        result = cur.fetchone()

        data['totalusage'] = float(result['POWER_USAGE'])
        data['cost'] = get_power_cost(username, db)

        data['usagcourse'] = get_device_energy_course(username, room, devicetype, deviceid, von, bis, db)

        return {'energydata': data}

def get_device_energy_course(username, room, type, id, von, bis, db):
    course = []

    with db:
        cur = db.cursor()

        cur.execute("SELECT POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM DEVICE_ENERGY_DATA WHERE DEVICE_TYPE = :type AND DEVICE_ID = :id AND DATE >= :von AND DATE <= :bis ORDER BY DATE ASC",
            {'type': type, 'id': id, 'von': von, 'bis': bis})

        for item in cur.fetchall():
            course.append({'date': item['DATETIME'], 'usage': item['POWER_USAGE']})

        return course

def get_energy_course(username, room, von, bis, db):
    course = []

    with db:
        cur = db.cursor()

        if (room is not None and room != ""):
            cur.execute("SELECT POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM ROOM_ENERGY_DATA WHERE ROOM_ID = :room AND DATE >= :von AND DATE <= :bis ORDER BY DATE ASC", {'room': room, 'von': von, 'bis': bis})
        else:
            cur.execute("SELECT SUM(POWER_USAGE) as POWER_USAGE, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM ROOM_ENERGY_DATA WHERE DATE >= :von AND DATE <= :bis GROUP BY DATE ORDER BY DATE ASC", {'von': von, 'bis': bis})

        for item in cur.fetchall():
            course.append({'date': item['DATETIME'], 'usage': item['POWER_USAGE']})

    return course

def set_power_cost(username, cost, db):
    return Status(type=STATUS_OK).get_dict()

def get_power_cost(username, db):
    power_cost = 0.3

    return round(power_cost, 2)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.items.ShoppingListItem import ShoppingListItem
from homevee.items.Status import *


def get_shopping_list(username, db):
    items = ShoppingListItem.load_all(db)

    #with db:
    #    cur = db.cursor()
    #    cur.execute("SELECT * FROM SHOPPING_LIST")
    #    for item in cur.fetchall():
    #        if item['AMOUNT'] is None:
    #            amount = -1
    #        else:
    #            amount = int(item['AMOUNT'])
    #        items.append({'id': item['ID'], 'amount': amount, 'item': item['ITEM']})
    #    cur.close()

    return {'items': ShoppingListItem.list_to_dict(items)}

def add_edit_shopping_list_item(username, id, amount, name, db):
    #Bearbeiten oder Hinzufügen?

    with db:

        cur = db.cursor()

        if id is not "" and id is not None:
            #Add to list
            cur.execute("UPDATE OR IGNORE SHOPPING_LIST SET AMOUNT = :amount, ITEM = :name WHERE ID = :id",
                        {'amount': amount, 'name': name, 'id': id})
        else:
            #Edit list entry

            #Check if entry with same name exists and notify user(???)

            cur.execute("INSERT OR IGNORE INTO SHOPPING_LIST (AMOUNT, ITEM) VALUES (:amount, :name);",
                        {'amount': amount, 'name': name})

            #TODO add generated id to object

    #Existiert bereits? Menge addieren?

    return Status(type=STATUS_OK).get_dict().get_dict()

def delete_shopping_list_item(user, id, db):
    item = ShoppingListItem.load_from_db(id, db)

    result = item.delete()

    #Abfrage erfolgreich?
    if result:
        return Status(type=STATUS_OK).get_dict().get_dict()
    else:
        return {'result': 'noadmin'}


import traceback

from homevee.DeviceAPI.zwave.utils import do_zwave_request
from homevee.Helper import Logger
from homevee.exceptions import DatabaseSaveFailedException, ItemNotFoundException, InvalidParametersException
from homevee.items.Device.Thermostat import Thermostat
from homevee.utils import database
from homevee.utils.device_types import ZWAVE_THERMOSTAT


class ZWaveThermostat(Thermostat):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(ZWaveThermostat, self).__init__(name, icon, location, id=id, temp=temp)

    def get_device_type(self):
        return ZWAVE_THERMOSTAT

    def delete(self, db=None):
        try:
            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM ZWAVE_THERMOSTATS WHERE ID == :id'", {'id': self.id})

                cur.close()
            return True
        except:
            return False

    def save_to_db(self, db=None):
        try:
            if (db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                # insert
                cur.execute("""INSERT OR IGNORE INTO ZWAVE_THERMOSTATS (THERMOSTAT_ID, NAME, ICON, VALUE, RAUM) VALUES 
                            (:id, :name, :icon, :last_temp, :room)""",
                            {'id': self.id, 'name': self.name, 'icon': self.icon,
                             'last_temp': self.temp, 'room': self.location})
                # update
                cur.execute("""UPDATE OR IGNORE ZWAVE_THERMOSTATS SET NAME = :name, ICON = :icon,
                            VALUE = :last_temp, RAUM = :room WHERE THERMOSTAT_ID = :id""",
                            {'name': self.name, 'icon': self.icon, 'last_temp': self.temp,
                             'room': self.location, 'id': self.id})

                cur.close()

                # TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save max-thermostat to database")

    def build_dict(self):
        dict = {
            'name': self.name,
            'icon': self.icon,
            'id': self.id,
            'temp': self.temp,
            'location': self.location
        }
        return dict

    def update_temp(self, temp, db=None):
        try:
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + str(self.id)
                                      + "/command/exact?level=" + str(temp),db)
            if result is not None and result['code'] == 200:
                return True
            else:
                return False
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return False

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return ZWaveThermostat.load_all_from_db('SELECT * FROM MAX_THERMOSTATS WHERE ID IN (%s)'
                                              % ','.join('?' * len(ids)), ids, db)

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)
            for result in cur.fetchall():
                item = ZWaveThermostat(result['NAME'], result['ICON'], result['RAUM'], result['THERMOSTAT_ID'],
                                     result['VALUE'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def load_from_db(id, db=None):
        items = ZWaveThermostat.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find zwave-thermostat-id: " + id)
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return ZWaveThermostat.load_all_from_db('SELECT * FROM ZWAVE_THERMOSTATS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            name = dict['name']
            id = dict['id']
            location = dict['location']
            temp = dict['temp']
            icon = dict['icon']

            item = ZWaveThermostat(name, icon, location, id, temp)

            return item

        except:
            raise InvalidParametersException("Invalid parameters for ZWaveThermostat.create_from_dict()")

    @staticmethod
    def get_all(location=None, db=None):
        devices = []

        all_devices = ZWaveThermostat.load_all(db)

        if location is None or location == "all":
            # get all devices of all types
            return all_devices

        for device in all_devices:
            if str(device.location) == str(location.id):
                devices.append(device)

        return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.request, urllib.error, urllib.parse

from homevee.Helper import Logger


def get_recipe(id):
    url = "https://api.chefkoch.de/v2/recipes/"+str(id)

    result = urllib.request.urlopen(url).read()

    data = json.loads(result)

    return data

def search_recipes(keyword, limit=None, offset=None, min_rating=None, max_time=None):
    keyword = keyword.replace(" ", "%20")

    url = "https://api.chefkoch.de/v2/recipes?query=" + str(keyword)

    if limit is not None:
        url = url + "&limit=" + str(limit)

    if offset is not None:
        url = url + "&offset=" + str(offset)

    if min_rating is not None:
        url = url + "&minimumRating=" + str(min_rating)

    if max_time is not None:
        url = url + "&maximumTime=" + str(max_time)

    Logger.log(url)

    result = urllib.request.urlopen(url).read()

    result = json.loads(result)

    recipes = result['results']

    return recipes


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
from homevee.Functions.condition_actions.actions import run_actions
from homevee.Functions.condition_actions.conditions import conditions_true
from homevee.Helper import Logger

class Automation():
    def run_trigger_automation(self, trigger_type, type, id, value, db):
        return

    def run_automations(self, automations, db):
        for item in automations:
            Logger.log(item)
            if (conditions_true(json.loads(item['CONDITION_DATA']), db)):
                if (int(item['TRIGGERED']) == 0):
                    Logger.log("running item: "+str(item))
                    run_actions(json.loads(item['ACTION_DATA']), db)
                    self.set_triggered(item['ID'], 1, db)
            else:
                self.set_triggered(item['ID'], 0, db)

    def set_triggered(self, id, triggered, db):
        with db:
            cur = db.cursor()

            cur.execute("UPDATE AUTOMATION_DATA SET TRIGGERED = :triggered WHERE ID = :id",
                        {'triggered': triggered, 'id': id})


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback
import urllib.error
import urllib.parse
import urllib.request

from homevee.DeviceAPI import zwave
from homevee.Helper import Logger
from homevee.Helper.helper_functions import has_permission
from homevee.items.Device.Switch.Funksteckdose import Funksteckdose
from homevee.items.Status import *
from homevee.utils.device_types import *


def set_modes(username, type, id, mode, db, check_user=True):
    if type == FUNKSTECKDOSE:
        return set_socket(username, id, mode, db, check_user)
    elif type == ZWAVE_SWITCH:
        return set_zwave_switch(username, id, mode, db, check_user)
    elif type == URL_SWITCH:
        return set_url_switch_binary(username, id, mode, db, check_user)
    elif type == URL_TOGGLE:
        return set_url_toggle(username, id, db, check_user)
    else:
        raise ValueError("Type does not exist")

def set_url_switch_binary(username, id, mode, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM URL_SWITCH_BINARY WHERE ID == :id",
                    {'id': id})

        data = cur.fetchone()

        if check_user and not has_permission(username, data['LOCATION'], db):
            return {'result': 'nopermission'}

        if mode == "":
            urllib.request.urlopen(data['URL_ON'])
        else:
            urllib.request.urlopen(data['URL_OFF'])

    return Status(type=STATUS_OK).get_dict()

def set_url_toggle(username, id, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM URL_TOGGLE WHERE ID == :id",
                    {'id': id})

        data = cur.fetchone()

        if check_user and not has_permission(username, data['LOCATION'], db):
            return {'result': 'nopermission'}

        urllib.request.urlopen(data['TOGGLE_URL'])

    return Status(type=STATUS_OK).get_dict()

def set_socket(user, id, mode, db, check_user=True):
    try:
        funksteckdose = Funksteckdose.load_from_db(id, db)

        if check_user and not user.has_permission(funksteckdose.location):
            return {'result': 'nopermission'}

        if(funksteckdose.set_mode(mode, db)):
            return Status(type=STATUS_OK).get_dict()
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
    return Status(type=STATUS_ERROR).get_dict()

def set_zwave_switch(username, id, mode, db, check_user=True):
    try:
        with db:
            cur = db.cursor()
            cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE ID == :device",
                        {'device': id})

            data = cur.fetchone()

            if check_user and not has_permission(username, data['LOCATION'], db):
                return {'result': 'nopermission'}

            device_id = data['ID']

            state = "on" if (int(mode) == 1) else "off"

            result = zwave.device_control.set_binary_device(device_id, state, db)

            if result['code'] == 200:
                cur.execute("UPDATE ZWAVE_SWITCHES SET VALUE = :value WHERE ID = :id",
                            {'value': mode, 'id': id})

                return Status(type=STATUS_OK).get_dict()
    except Exception as e:
        if Logger.IS_DEBUG:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
        return Status(type=STATUS_ERROR).get_dict()

def set_diy_switch(id, mode, db, check_user=True):
    return


#!/usr/bin/python
# -*- coding: utf-8 -*-

from pymax.cube import Cube

from homevee.Helper import Logger
from homevee.items.Gateway import Gateway
from homevee.items.Status import *


def get_devices(ip):
    thermostats = []
    try:
        with Cube(ip) as cube:
            for room in cube.rooms:
                for device in room.devices:
                    Logger.log(device)
                    thermostat = {'title': device.name, 'id': device.serial, 'room': room.name}

                    thermostats.append(thermostat)
    except:
        thermostats = []

    return thermostats


def set_temp(id, value, db):
    gateway = Gateway.load_from_db("MAX! Cube", db)

    try:
        with Cube(gateway.ip) as cube:
            for room in cube.rooms:
                for device in room.devices:
                    if id == device.serial:
                        result = cube.set_mode_manual(room.room_id, room.rf_address, float(value))

                        Logger.log(result)

                        with db:
                            cur = db.cursor()

                            cur.execute("UPDATE MAX_THERMOSTATS SET LAST_TEMP = :temp WHERE ID == :id",
                                        {'temp': value, 'id': id})

                        return Status(type=STATUS_OK).get_dict()
    except:
        return Status(type=STATUS_ERROR).get_dict()


import json

from homevee.items import Item
from homevee.items.Room import Room


class Permission(Item):
    def __init__(self, permission_key, name=None):
        super(Permission, self).__init__()

        if name is not None:
            self.name = name
        else:
            if permission_key == "admin":
                self.name = "Administrator"
            else:
                self.name = Room.get_name_by_id(permission_key, None)

        self.key = permission_key

    def build_dict(self):
        dict = {
            'key': self.key,
            'name': self.name
        }
        return dict

    @staticmethod
    def create_list_from_json(json_string):
        data = json.loads(json_string)

        permissions = []

        for item in data['permissions']:
            permissions.append(Permission(item))

        return permissions

    @staticmethod
    def list_to_dict(list):
        permissions = []

        for item in list:
            permissions.append(item.get_dict())

        return permissions

    @staticmethod
    def list_from_dict(dict):
        permissions = []

        for item in dict:
            permissions.append(Permission(item['key'], item['name']))

        return permissions


    @staticmethod
    def get_json_list(permissions):
        data = []

        for permission in permissions:
            data.append(permission.key)

        permission_data = {'permissions': data}

        return json.dumps(permission_data)


from homevee.exceptions import AbstractFunctionCallException
from homevee.items.Device import Device


class Thermostat(Device):
    def __init__(self, name, icon, location, id=None, temp=None):
        super(Thermostat, self).__init__(name, icon, location, id=id)
        self.temp = temp

    def set_temp(self, temp, db=None):
        if(self.update_temp(temp, db)):
            self.mode = temp
            self.save_to_db(db)
            print("result == True")
            return True
        print("result == False")
        return False

    def update_temp(self, temp, db=None):
        raise AbstractFunctionCallException("Switch.update_mode() is abstract")

    #@staticmethod
    #def get_all(location=None, db=None):
    #    # get all devices of all types
    #    devices = []
    #    devices.extend(Switch.get_all(location, db))
    #    return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.items.Device.Sensor.ZWaveSensor import *
from homevee.items.Room import Room
from homevee.items.Status import *
from homevee.utils.device_types import *

SENSOR_TYPE_MAP = {
    'temp': {'name': 'Temperatur', 'einheit': '°C', 'einheit_word': 'Grad'},
    'hygro': {'name': 'Luftfeuchtigkeit', 'einheit': '%', 'einheit_word': '%'},
    'helligkeit': {'name': 'Helligkeit', 'einheit': 'Lux', 'einheit_word': 'Lumen'},
    'uv': {'name': 'UV-Licht', 'einheit': 'UV-Index', 'einheit_word': 'UV-Index'},
    'powermeter': {'name': 'Stromverbrauch', 'einheit': 'Watt', 'einheit_word': 'Watt'},
}


def get_einheit(type, id, db):
    with db:
        cur = db.cursor()
        db_data = {
            ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'sensor_type_col': 'SENSOR_TYPE', 'id_col': 'ID'},
            MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'sensor_type_col': 'TYPE', 'id_col': 'ID'}
        }

        cur.execute("SELECT " + db_data[type]['sensor_type_col'] + " FROM " + db_data[type]['table'] + " WHERE " +
                    db_data[type]['id_col'] + " = :id",
                    {'id': id})

        return SENSOR_TYPE_MAP[cur.fetchone()[db_data[type]['sensor_type_col']]]['einheit']


def get_sensor_value(type, id, show_einheit, db):
    # TODO Klassen für Sensor-Typen erweitern

    if type == ZWAVE_SENSOR:
        sensor = ZWaveSensor.load_from_db(id, db)
    # elif type == ZWAVE_THERMOSTAT:
    #	sensor = ZWaveThermostat.load_from_db(id, db)
    # elif type == RADEMACHER_THERMOSTAT:
    #	sensor = RademacherThermostat.load_from_db(id, db)
    else:
        return "N/A"

    output = str(sensor.value)

    if show_einheit:
        output += " " + sensor.get_einheit()

    return output


def get_sensor_data(user, room, type, id, show_einheit, db):
    if not user.has_permission(room):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    values = []

    for room in Room.load_all(db):
        value_array = []

        if not user.has_permission(room.id):
            continue

        devices = {}

        # ZWave Sensoren
        devices[ZWAVE_THERMOSTAT] = ZWaveSensor.get_all(room, db)

        # TODO weitere Sensortypen

        # Iterate through all collected sensors

        for device_type in devices:
            for device in devices[device_type]:
                item = device.get_dict()
                value_array.append(item)

        value_item = {'name': room.name, 'location': room.id, 'icon': room.icon,
                      'value_array': value_array}

        values.append(value_item)

    return {'values': values}



from homevee.exceptions import AbstractFunctionCallException, InvalidParametersException
from homevee.items import Item


class Device(Item):
    def __init__(self, name, icon, location, id=None):
        super(Device, self).__init__()

        self.id = id
        self.name = name
        self.icon = icon
        self.location = location

    def get_device_type(self):
        raise AbstractFunctionCallException("Device.get_device_type() is abstract")

    def get_dict(self, fields=None):
        output_dict = {}

        dict = self.build_dict()

        if (fields is None):
            output_dict = dict
        else:
            try:
                for field in fields:
                    output_dict[field] = dict[field]
            except:
                raise InvalidParametersException("InvalidParams given for get_dict()")

        output_dict['device_type'] = self.get_device_type()

        return output_dict


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.Manager.api_key import get_api_key
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_GET_MOVIE_RATING

class VoiceMovieApiModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_GET_MOVIE_RATING

    def get_label(self):
        return "getmovierating"

    def run_command(self, username, text, context, db):
        return self.get_movie_rating(username, text, context, db)

    def get_movie_rating(self, username, text, context, db):
        api_key = get_api_key('TMDB', db)

        words = text.split(" ")

        query = None

        is_query = False

        film_index = -1

        for i in range(0, len(words)):
            word = words[i]
            if is_query:
                if query is None:
                    query = word.capitalize()
                else:
                    query = query + " " + word.capitalize()

            if word == "film":
                if not is_query:
                    film_index = i
                is_query = True

            if is_query and i > film_index and query != None:
                if word == "gut" or word == "schlecht":
                    break

        #url = 'https://api.themoviedb.org/3/movie/'+movie_id+'?api_key='+api_key+'&language=de/'
        #response = urllib2.urlopen(url).read()
        #print(response)

        output = "getmovierating"

        return {'msg_speech': output, 'msg_text': output}


from homevee.items.Device import Device

SENSOR_TYPE_MAP = {
    'temp': {'name': 'Temperatur', 'einheit': '°C', 'einheit_word': 'Grad'},
    'hygro': {'name': 'Luftfeuchtigkeit', 'einheit': '%', 'einheit_word': '%'},
    'helligkeit': {'name': 'Helligkeit', 'einheit': 'Lux', 'einheit_word': 'Lumen'},
    'uv': {'name': 'UV-Licht', 'einheit': 'UV-Index', 'einheit_word': 'UV-Index'},
    'powermeter': {'name': 'Stromverbrauch', 'einheit': 'Watt', 'einheit_word': 'Watt'},
}

class Sensor(Device):
    def __init__(self, name, icon, location, save_data, sensor_type, id=None, value=None):
        super(Sensor, self).__init__(name, icon, location, id=id)
        self.save_data = save_data
        self.sensor_type = sensor_type

        if value is None:
            value = "N/A"
        else:
            self.value = value

    def get_einheit(self):
        return SENSOR_TYPE_MAP[self.sensor_type]['einheit']


import os
import traceback

from passlib.handlers.pbkdf2 import pbkdf2_sha512

from homevee.Helper import Logger
from homevee.exceptions import ItemNotFoundException, InvalidParametersException, DatabaseSaveFailedException
from homevee.items import Item
from homevee.items.User.Permission import Permission
from homevee.utils import database


class User(Item):
    def __init__(self, username, hashed_password, ip, at_home, events_last_checked,
                 permissions, dashboard_data, salt, fcm_token, current_location, id=None):
        super(User, self).__init__()

        self.id = id
        self.username = username
        self.hashed_password = hashed_password
        self.ip = ip
        self.at_home = at_home
        self.events_last_checked = events_last_checked
        self.permissions = permissions
        self.dashboard_data = dashboard_data
        self.salt = salt
        self.fcm_token = fcm_token
        self.current_location = current_location

    def delete(self, db=None):
        try:
            with db:
                cur = db.cursor()
                cur.execute("DELETE FROM USERDATA WHERE USERNAME == :user'", {'user': self.username})

                cur.close()
            return True
        except:
            return False

    def has_permission(self, permission):
        for user_permission in self.permissions:
            if user_permission.key == "admin" or user_permission.id == permission:
                return True

        return False

    @staticmethod
    def hash_password(password):
        salt = os.urandom(12).hex()

        hashed_pw = pbkdf2_sha512.encrypt(password + salt, rounds=200000)

        # print "Password: "+password
        # print "Salt: "+salt
        # print "Hashed: "+hashed_pw

        #self.hashed_password = hashed_pw
        #self.salt = salt

        return (hashed_pw, salt)

    def verify(self, password):
        try:
            return pbkdf2_sha512.verify(password + self.salt, self.hashed_password)
        except:
            return False

    def save_to_db(self, db=None):
        try:
            if (db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                # insert
                if (self.id is None or self.id == ""):
                    cur.execute("""INSERT INTO USERDATA (USERNAME, PASSWORD, IP, PERMISSIONS, PW_SALT) VALUES 
                                (:username, :password, :ip, :permissions, :salt)""",
                                {'username': self.username, 'password': self.hashed_password, 'ip': self.ip,
                                 'permissions': Permission.get_json_list(self.permissions), 'salt': self.salt})
                # update
                else:
                    cur.execute("""UPDATE USERDATA SET PASSWORD = :password, IP = :ip, PERMISSIONS = :permissions,
                                PW_SALT = :salt WHERE ID = :id""",
                                {'password': self.hashed_password, 'ip': self.ip,
                                 'permissions': Permission.get_json_list(self.permissions),
                                 'salt': self.salt, 'id': self.id})

                cur.close()

                #TODO add generated id to object
        except:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            raise DatabaseSaveFailedException("Could not save user to database")

    def build_dict(self):
        dict = {
            'id': self.id,
            'username': self.username,
            'password': self.hashed_password,
            'ip': self.ip,
            'at_home': self.at_home,
            'events_last_checked': self.events_last_checked,
            'permissions': Permission.list_to_dict(self.permissions),
            'dashboard_data': self.dashboard_data,
            'salt': self.salt,
            'fcm_token': self.fcm_token,
            'current_location': self.current_location
        }
        return dict

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)
            for result in cur.fetchall():
                item = User(result['USERNAME'], result['PASSWORD'], result['IP'], result['AT_HOME'],
                            result['EVENTS_LAST_CHECKED'], Permission.create_list_from_json(result['PERMISSIONS']),
                            result['DASHBOARD_DATA'], result['PW_SALT'], result['FCM_TOKEN'],
                            result['CURRENT_LOCATION'], result['ID'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def load_by_permission(permission, db=None):
        users = User.load_all(db)

        output = []

        for user in users:
            if user.has_permission(permission):
                output.append(user)

        return users

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return User.load_all_from_db('SELECT * FROM USERDATA WHERE ID IN (%s)' % ','.join('?'*len(ids)),
                                     ids, db)

    @staticmethod
    def load_all_usernames_from_db(usernames, db=None):
        return User.load_all_from_db('SELECT * FROM USERDATA WHERE USERNAME IN (%s)' % ','.join('?'*len(usernames)),
                                     usernames, db)

    @staticmethod
    #ID is username
    def load_from_db(id, db=None):
        items = User.load_all_ids_from_db([id], db)

        if ((len(items) == 0) or (items[0].id != id)):
            raise ItemNotFoundException("Could not find user-id: " + id)
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return User.load_all_from_db('SELECT * FROM USERDATA', {}, db)

    @staticmethod
    #ID is username
    def load_username_from_db(username, db=None):
        users = User.load_all_usernames_from_db([username], db)

        if((len(users) == 0) or (users[0].username != username)):
            raise ItemNotFoundException("Could not find username: " + username)
        else:
            return users[0]

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            username = dict['username']
            password = dict['password']
            ip = dict['ip']
            at_home = dict['at_home']
            events_last_checked = dict['events_last_checked']
            permissions = Permission.list_from_dict(dict['permissions'])
            dashboard_data = dict['dashboard_data']
            salt = dict['salt']
            fcm_token = dict['fcm_token']
            current_location = dict['current_location']

            user = User(username, password, ip, at_home, events_last_checked,
                        permissions, dashboard_data, salt, fcm_token, current_location, id)

            return user
        except:
            raise InvalidParametersException("User.create_from_dict(): invalid dict")